@@ -17,7 +17,10 @@ public:
 	std::vector<const battle::Unit *> unreachableEnemies;
 
 	PotentialTargets(){};
-	PotentialTargets(const battle::Unit * attacker, const HypotheticBattle & state);
+	PotentialTargets(
+		const battle::Unit * attacker,
+		DamageCache & damageCache,
+		std::shared_ptr<HypotheticBattle> hb);
 
 	const AttackPossibility & bestAction() const;
 	int64_t bestActionValue() const;
