@@ -8,6 +8,7 @@
  *
  */
 
+#include "StdInc.h"
 #include "utils.h"
 
 #import <UIKit/UIKit.h>
@@ -43,4 +44,10 @@ void hideLoadingIndicator()
 	[indicator removeFromSuperview];
 	indicator = nil;
 }
+
+void hapticFeedback()
+{
+    auto hapticGen = [[UIImpactFeedbackGenerator alloc] initWithStyle:UIImpactFeedbackStyleLight];
+    [hapticGen impactOccurred];
+}
 }
