@@ -12,25 +12,27 @@
 
 #include "Buttons.h"
 #include "TextControls.h"
+#include "RadialMenu.h"
 
 #include "../gui/CGuiHandler.h"
-
-#include "../CGameInfo.h"
-#include "../CPlayerInterface.h"
+#include "../gui/WindowHandler.h"
+#include "../render/IImage.h"
+#include "../render/Graphics.h"
 #include "../windows/CCreatureWindow.h"
 #include "../windows/GUIClasses.h"
+#include "../CGameInfo.h"
+#include "../CPlayerInterface.h"
 
 #include "../../CCallback.h"
 
+#include "../../lib/ArtifactUtils.h"
 #include "../../lib/CGeneralTextHandler.h"
 #include "../../lib/CCreatureHandler.h"
+#include "../../lib/CConfigHandler.h"
 #include "../../lib/mapObjects/CGHeroInstance.h"
-
-#include "../../lib/CGameState.h"
-
-#ifdef VCMI_MAC
-#define SDL_SCANCODE_LCTRL SDL_SCANCODE_LGUI
-#endif
+#include "../../lib/networkPacks/ArtifactLocation.h"
+#include "../../lib/TextOperations.h"
+#include "../../lib/gameState/CGameState.h"
 
 void CGarrisonSlot::setHighlight(bool on)
 {
@@ -53,18 +55,18 @@ void CGarrisonSlot::hover (bool on)
 				if(owner->getSelection() == this)
 				{
 					temp = CGI->generaltexth->tcommands[4]; //View %s
-					boost::algorithm::replace_first(temp,"%s",creature->nameSing);
+					boost::algorithm::replace_first(temp,"%s",creature->getNameSingularTranslated());
 				}
 				else if (owner->getSelection()->creature == creature)
 				{
 					temp = CGI->generaltexth->tcommands[2]; //Combine %s armies
-					boost::algorithm::replace_first(temp,"%s",creature->nameSing);
+					boost::algorithm::replace_first(temp,"%s",creature->getNameSingularTranslated());
 				}
 				else if (owner->getSelection()->creature)
 				{
 					temp = CGI->generaltexth->tcommands[7]; //Exchange %s with %s
-					boost::algorithm::replace_first(temp,"%s",owner->getSelection()->creature->nameSing);
-					boost::algorithm::replace_first(temp,"%s",creature->nameSing);
+					boost::algorithm::replace_first(temp,"%s",owner->getSelection()->creature->getNameSingularTranslated());
+					boost::algorithm::replace_first(temp,"%s",creature->getNameSingularTranslated());
 				}
 				else
 				{
@@ -75,16 +77,16 @@ void CGarrisonSlot::hover (bool on)
 			}
 			else
 			{
-				const bool isHeroOnMap = owner->armedObjs[0] // Hero is not a visitor and not a garrison defender
-					&& owner->armedObjs[0]->ID == Obj::HERO
-					&& (!owner->armedObjs[1] || owner->armedObjs[1]->ID == Obj::HERO) // one hero or we are in the Heroes exchange window
-					&& !(static_cast<const CGHeroInstance*>(owner->armedObjs[0]))->inTownGarrison;
+				const bool isHeroOnMap = owner->upperArmy() // Hero is not a visitor and not a garrison defender
+					&& owner->upperArmy()->ID == Obj::HERO
+					&& (!owner->lowerArmy() || owner->lowerArmy()->ID == Obj::HERO) // one hero or we are in the Heroes exchange window
+					&& !(static_cast<const CGHeroInstance*>(owner->upperArmy()))->inTownGarrison;
 
 				if(isHeroOnMap)
 				{
 					temp = CGI->generaltexth->allTexts[481]; //Select %s
 				}
-				else if(upg == EGarrisonType::UP)
+				else if(upg == EGarrisonType::UPPER)
 				{
 					temp = CGI->generaltexth->tcommands[12]; //Select %s (in garrison)
 				}
@@ -92,7 +94,7 @@ void CGarrisonSlot::hover (bool on)
 				{
 					temp = CGI->generaltexth->tcommands[32]; //Select %s (visiting)
 				}
-				boost::algorithm::replace_first(temp,"%s",creature->nameSing);
+				boost::algorithm::replace_first(temp,"%s",creature->getNameSingularTranslated());
 			}
 		}
 		else
@@ -110,7 +112,7 @@ void CGarrisonSlot::hover (bool on)
 				else
 				{
 					temp = CGI->generaltexth->tcommands[6]; //Move %s
-					boost::algorithm::replace_first(temp,"%s",owner->getSelection()->creature->nameSing);
+					boost::algorithm::replace_first(temp,"%s",owner->getSelection()->creature->getNameSingularTranslated());
 				}
 			}
 			else
@@ -118,26 +120,24 @@ void CGarrisonSlot::hover (bool on)
 				temp = CGI->generaltexth->tcommands[11]; //Empty
 			}
 		}
-		GH.statusbar->setText(temp);
+		GH.statusbar()->write(temp);
 	}
 	else
 	{
-		GH.statusbar->clear();
+		GH.statusbar()->clear();
 	}
 }
 
 const CArmedInstance * CGarrisonSlot::getObj() const
 {
-	return 	owner->armedObjs[upg];
+	return owner->army(upg);
 }
 
-/// @return Whether the unit in the slot belongs to the current player.
 bool CGarrisonSlot::our() const
 {
-	return owner->owned[upg];
+	return owner->isArmyOwned(upg);
 }
 
-/// @return Whether the unit in the slot belongs to an ally but not to the current player.
 bool CGarrisonSlot::ally() const
 {
 	if(!getObj())
@@ -163,9 +163,9 @@ std::function<void()> CGarrisonSlot::getDismiss() const
 bool CGarrisonSlot::viewInfo()
 {
 	UpgradeInfo pom;
-	LOCPLINT->cb->getUpgradeInfo(getObj(), ID, pom);
+	LOCPLINT->cb->fillUpgradeInfo(getObj(), ID, pom);
 
-	bool canUpgrade = getObj()->tempOwner == LOCPLINT->playerID && pom.oldID>=0; //upgrade is possible
+	bool canUpgrade = getObj()->tempOwner == LOCPLINT->playerID && pom.oldID != CreatureID::NONE; //upgrade is possible
 	std::function<void(CreatureID)> upgr = nullptr;
 	auto dism = getDismiss();
 	if(canUpgrade) upgr = [=] (CreatureID newID) { LOCPLINT->cb->upgradeCreature(getObj(), ID, newID); };
@@ -177,7 +177,7 @@ bool CGarrisonSlot::viewInfo()
 		elem->block(true);
 
 	redraw();
-	GH.pushIntT<CStackWindow>(myStack, dism, pom, upgr);
+	GH.windows().createAndPushWindow<CStackWindow>(myStack, dism, pom, upgr);
 	return true;
 }
 
@@ -186,28 +186,28 @@ bool CGarrisonSlot::viewInfo()
 bool CGarrisonSlot::highlightOrDropArtifact()
 {
 	bool artSelected = false;
-	if (CWindowWithArtifacts* chw = dynamic_cast<CWindowWithArtifacts*>(GH.topInt().get())) //dirty solution
+	if (auto chw = GH.windows().topWindow<CWindowWithArtifacts>()) //dirty solution
 	{
-		std::shared_ptr<CArtifactsOfHero::SCommonPart> commonInfo = chw->getCommonPart();
-		const CArtifactInstance * art = nullptr;
-		if(commonInfo)
-			art = commonInfo->src.art;
+		const auto pickedArtInst = chw->getPickedArtifact();
 
-		if(art)
+		if(pickedArtInst)
 		{
-			const CGHeroInstance *srcHero = commonInfo->src.AOH->getHero();
+			const auto * srcHero = chw->getHeroPickedArtifact();
 			artSelected = true;
 			if (myStack) // try dropping the artifact only if the slot isn't empty
 			{
-				ArtifactLocation src(srcHero, commonInfo->src.slotID);
-				ArtifactLocation dst(myStack, ArtifactPosition::CREATURE_SLOT);
-				if (art->canBePutAt(dst, true))
+				ArtifactLocation src(srcHero->id, ArtifactPosition::TRANSITION_POS);
+				ArtifactLocation dst(getObj()->id, ArtifactPosition::CREATURE_SLOT);
+				dst.creature = getSlot();
+
+				if(pickedArtInst->canBePutAt(myStack, ArtifactPosition::CREATURE_SLOT, true))
 				{	//equip clicked stack
-					if(dst.getArt())
+					if(auto dstArt = myStack->getArt(ArtifactPosition::CREATURE_SLOT))
 					{
 						//creature can wear only one active artifact
 						//if we are placing a new one, the old one will be returned to the hero's backpack
-						LOCPLINT->cb->swapArtifacts(dst, ArtifactLocation(srcHero, dst.getArt()->firstBackpackSlot(srcHero)));
+						LOCPLINT->cb->swapArtifacts(dst, ArtifactLocation(srcHero->id,
+							ArtifactUtils::getArtBackpackPosition(srcHero, dstArt->getTypeId())));
 					}
 					LOCPLINT->cb->swapArtifacts(src, dst);
 				}
@@ -232,8 +232,6 @@ bool CGarrisonSlot::highlightOrDropArtifact()
 bool CGarrisonSlot::split()
 {
 	const CGarrisonSlot * selection = owner->getSelection();
-	owner->p2 = ID;   // store the second stack pos
-	owner->pb = upg;  // store the second stack owner (up or down army)
 	owner->setSplittingMode(false);
 
 	int minLeft=0, minRight=0;
@@ -257,17 +255,15 @@ bool CGarrisonSlot::split()
 	int countLeft = selection->myStack ? selection->myStack->count : 0;
 	int countRight = myStack ? myStack->count : 0;
 
-	GH.pushIntT<CSplitWindow>(selection->creature, std::bind(&CGarrisonInt::splitStacks, owner, _1, _2),
-		minLeft, minRight, countLeft, countRight);
+	auto splitFunctor = [this, selection](int amountLeft, int amountRight)
+	{
+		owner->splitStacks(selection, owner->army(upg), ID, amountRight);
+	};
+
+	GH.windows().createAndPushWindow<CSplitWindow>(selection->creature,  splitFunctor, minLeft, minRight, countLeft, countRight);
 	return true;
 }
 
-/// If certain creates cannot be moved, the selection should change
-/// Force reselection in these cases
-///     * When attempting to take creatures from ally
-///     * When attempting to swap creatures with an ally
-///     * When attempting to take unremovable units
-/// @return Whether reselection must be done
 bool CGarrisonSlot::mustForceReselection() const
 {
 	const CGarrisonSlot * selection = owner->getSelection();
@@ -282,26 +278,24 @@ bool CGarrisonSlot::mustForceReselection() const
 		return true;
 	if (!owner->removableUnits)
 	{
-		if (selection->upg == EGarrisonType::UP)
+		if (selection->upg == EGarrisonType::UPPER)
 			return true;
 		else
-			return creature || upg == EGarrisonType::UP;
+			return creature || upg == EGarrisonType::UPPER;
 	}
 	return false;
 }
 
-void CGarrisonSlot::clickRight(tribool down, bool previousState)
+void CGarrisonSlot::showPopupWindow(const Point & cursorPosition)
 {
-	if(creature && down)
+	if(creature)
 	{
-		GH.pushIntT<CStackWindow>(myStack, true);
+		GH.windows().createAndPushWindow<CStackWindow>(myStack, true);
 	}
 }
 
-void CGarrisonSlot::clickLeft(tribool down, bool previousState)
+void CGarrisonSlot::clickPressed(const Point & cursorPosition)
 {
-	if(down)
-	{
 		bool refr = false;
 		const CGarrisonSlot * selection = owner->getSelection();
 
@@ -325,7 +319,7 @@ void CGarrisonSlot::clickLeft(tribool down, bool previousState)
 		}
 		else
 		{
-			const CArmedInstance * selectedObj = owner->armedObjs[selection->upg];
+			const CArmedInstance * selectedObj = owner->army(selection->upg);
 			bool lastHeroStackSelected = false;
 			if(selectedObj->stacksCount() == 1
 				&& owner->getSelection()->upg != upg
@@ -334,19 +328,19 @@ void CGarrisonSlot::clickLeft(tribool down, bool previousState)
 				lastHeroStackSelected = true;
 			}
 
-			if((owner->getSplittingMode() || LOCPLINT->shiftPressed()) // split window
+			if((owner->getSplittingMode() || GH.isKeyboardShiftDown()) // split window
 				&& (!creature || creature == selection->creature))
 			{
 				refr = split();
 			}
 			else if(!creature && lastHeroStackSelected) // split all except last creature
-				LOCPLINT->cb->splitStack(selectedObj, owner->armedObjs[upg], selection->ID, ID, selection->myStack->count - 1);
+				LOCPLINT->cb->splitStack(selectedObj, owner->army(upg), selection->ID, ID, selection->myStack->count - 1);
 			else if(creature != selection->creature) // swap
-				LOCPLINT->cb->swapCreatures(owner->armedObjs[upg], selectedObj, ID, selection->ID);
+				LOCPLINT->cb->swapCreatures(owner->army(upg), selectedObj, ID, selection->ID);
 			else if(lastHeroStackSelected) // merge last stack to other hero stack
 				refr = split();
 			else // merge
-				LOCPLINT->cb->mergeStacks(selectedObj, owner->armedObjs[upg], selection->ID, ID);
+				LOCPLINT->cb->mergeStacks(selectedObj, owner->army(upg), selection->ID, ID);
 		}
 		if(refr)
 		{
@@ -354,20 +348,52 @@ void CGarrisonSlot::clickLeft(tribool down, bool previousState)
 			hover(false);
 			hover(true);
 		}
-	}
+}
+
+void CGarrisonSlot::gesture(bool on, const Point & initialPosition, const Point & finalPosition)
+{
+	if(!on)
+		return;
+
+	if(!myStack)
+		return;
+
+	if (!settings["input"]["radialWheelGarrisonSwipe"].Bool())
+		return;
+
+	const auto * otherArmy = upg == EGarrisonType::UPPER ? owner->lowerArmy() : owner->upperArmy();
+
+	bool stackExists = myStack != nullptr;
+	bool hasSameUnit = stackExists && !owner->army(upg)->getCreatureSlots(myStack->type, ID).empty();
+	bool hasOwnEmptySlots = stackExists && owner->army(upg)->getFreeSlot() != SlotID();
+	bool exchangeMode = stackExists && owner->upperArmy() && owner->lowerArmy();
+	bool hasOtherEmptySlots = exchangeMode && otherArmy->getFreeSlot() != SlotID();
+	bool hasAnyEmptySlots = hasOtherEmptySlots || hasOwnEmptySlots;
+
+	std::vector<RadialMenuConfig> menuElements = {
+		{ RadialMenuConfig::ITEM_NW, hasSameUnit, "stackMerge", "vcmi.radialWheel.mergeSameUnit", [this](){owner->bulkMergeStacks(this);} },
+		{ RadialMenuConfig::ITEM_NE, hasOwnEmptySlots, "stackFillOne", "vcmi.radialWheel.fillSingleUnit", [this](){owner->bulkSplitStack(this);} },
+		{ RadialMenuConfig::ITEM_WW, hasOwnEmptySlots, "stackSplitOne", "vcmi.radialWheel.splitSingleUnit", [this](){splitIntoParts(this->getGarrison(), 1); } },
+		{ RadialMenuConfig::ITEM_EE, hasOwnEmptySlots, "stackSplitEqual", "vcmi.radialWheel.splitUnitEqually", [this](){owner->bulkSmartSplitStack(this);} },
+		{ RadialMenuConfig::ITEM_SW, hasOtherEmptySlots, "heroMove", "vcmi.radialWheel.moveUnit", [this](){owner->moveStackToAnotherArmy(this);} },
+		{ RadialMenuConfig::ITEM_SE, hasAnyEmptySlots, "heroSwap", "vcmi.radialWheel.splitUnit", [this](){ owner->selectSlot(this); owner->splitClick();} },
+	};
+
+	if (hasAnyEmptySlots || hasSameUnit)
+		GH.windows().createAndPushWindow<RadialMenu>(pos.center(), menuElements);
 }
 
 void CGarrisonSlot::update()
 {
 	if(getObj() != nullptr)
 	{
-		addUsedEvents(LCLICK | RCLICK | HOVER);
+		addUsedEvents(LCLICK | SHOW_POPUP | GESTURE | HOVER);
 		myStack = getObj()->getStackPtr(ID);
 		creature = myStack ? myStack->type : nullptr;
 	}
 	else
 	{
-		removeUsedEvents(LCLICK | RCLICK | HOVER);
+		removeUsedEvents(LCLICK | SHOW_POPUP | GESTURE | HOVER);
 		myStack = nullptr;
 		creature = nullptr;
 	}
@@ -378,7 +404,7 @@ void CGarrisonSlot::update()
 		creatureImage->setFrame(creature->getIconIndex());
 
 		stackCount->enable();
-		stackCount->setText(boost::lexical_cast<std::string>(myStack->count));
+		stackCount->setText(TextOperations::formatMetric(myStack->count, 4));
 	}
 	else
 	{
@@ -387,7 +413,7 @@ void CGarrisonSlot::update()
 	}
 }
 
-CGarrisonSlot::CGarrisonSlot(CGarrisonInt * Owner, int x, int y, SlotID IID, CGarrisonSlot::EGarrisonType Upg, const CStackInstance * creature_)
+CGarrisonSlot::CGarrisonSlot(CGarrisonInt * Owner, int x, int y, SlotID IID, EGarrisonType Upg, const CStackInstance * creature_)
 	: ID(IID),
 	owner(Owner),
 	myStack(creature_),
@@ -399,13 +425,14 @@ CGarrisonSlot::CGarrisonSlot(CGarrisonInt * Owner, int x, int y, SlotID IID, CGa
 	pos.x += x;
 	pos.y += y;
 
-	std::string imgName = owner->smallIcons ? "cprsmall" : "TWCRPORT";
+	AnimationPath imgName = AnimationPath::builtin(owner->smallIcons ? "cprsmall" : "TWCRPORT");
 
-	creatureImage = std::make_shared<CAnimImage>(imgName, 0);
+	creatureImage = std::make_shared<CAnimImage>(graphics->getAnimation(imgName), 0);
 	creatureImage->disable();
 
-	selectionImage = std::make_shared<CAnimImage>(imgName, 1);
+	selectionImage = std::make_shared<CAnimImage>(graphics->getAnimation(imgName), 1);
 	selectionImage->disable();
+	selectionImage->center(creatureImage->pos.center());
 
 	if(Owner->smallIcons)
 	{
@@ -418,29 +445,38 @@ CGarrisonSlot::CGarrisonSlot(CGarrisonInt * Owner, int x, int y, SlotID IID, CGa
 		pos.h = 64;
 	}
 
-	stackCount = std::make_shared<CLabel>(pos.w, pos.h, owner->smallIcons ? FONT_TINY : FONT_MEDIUM, BOTTOMRIGHT, Colors::WHITE);
+	int labelPosW = pos.w;
+	int labelPosH = pos.h;
+
+	if(Owner->layout == CGarrisonInt::ESlotsLayout::REVERSED_TWO_ROWS) //labels under icon
+	{
+		labelPosW = pos.w / 2 + 1;
+		labelPosH += 7;
+	}
+	ETextAlignment labelAlignment = Owner->layout == CGarrisonInt::ESlotsLayout::REVERSED_TWO_ROWS
+			? ETextAlignment::CENTER
+			: ETextAlignment::BOTTOMRIGHT;
+
+	stackCount = std::make_shared<CLabel>(labelPosW, labelPosH, owner->smallIcons ? FONT_TINY : FONT_MEDIUM, labelAlignment, Colors::WHITE);
 
 	update();
 }
 
-void CGarrisonSlot::splitIntoParts(CGarrisonSlot::EGarrisonType type, int amount)
+void CGarrisonSlot::splitIntoParts(EGarrisonType type, int amount)
 {
 	auto empty = owner->getEmptySlot(type);
 
 	if(empty == SlotID())
 		return;
 
-	owner->pb = type;
-	owner->p2 = empty;
-	owner->splitStacks(1, amount);
+	owner->splitStacks(this, owner->army(type), empty, amount);
 }
 
 bool CGarrisonSlot::handleSplittingShortcuts()
 {
-	const Uint8 * state = SDL_GetKeyboardState(NULL);
-	const bool isAlt = !!state[SDL_SCANCODE_LALT];
-	const bool isLShift = !!state[SDL_SCANCODE_LSHIFT];
-	const bool isLCtrl = !!state[SDL_SCANCODE_LCTRL];
+	const bool isAlt = GH.isKeyboardAltDown();
+	const bool isLShift = GH.isKeyboardShiftDown();
+	const bool isLCtrl = GH.isKeyboardCtrlDown();
 
 	if(!isAlt && !isLShift && !isLCtrl)
 		return false; // This is only case when return false
@@ -497,25 +533,39 @@ void CGarrisonInt::addSplitBtn(std::shared_ptr<CButton> button)
 void CGarrisonInt::createSlots()
 {
 	int distance = interx + (smallIcons ? 32 : 58);
-	for(int i=0; i<2; i++)
+	for(auto i : { EGarrisonType::UPPER, EGarrisonType::LOWER })
 	{
+		Point offset = garOffset * static_cast<int>(i);
+
 		std::vector<std::shared_ptr<CGarrisonSlot>> garrisonSlots;
 		garrisonSlots.resize(7);
-		if(armedObjs[i])
+		if(army(i))
 		{
-			for(auto & elem : armedObjs[i]->Slots())
+			for(auto & elem : army(i)->Slots())
 			{
-				garrisonSlots[elem.first.getNum()] = std::make_shared<CGarrisonSlot>(this, i*garOffset.x + (elem.first.getNum()*distance), i*garOffset.y, elem.first, static_cast<CGarrisonSlot::EGarrisonType>(i), elem.second);
+				garrisonSlots[elem.first.getNum()] = std::make_shared<CGarrisonSlot>(this, offset.x + (elem.first.getNum()*distance), offset.y, elem.first, i, elem.second);
 			}
 		}
-		for(int j=0; j<7; j++)
+		for(int j = 0; j < 7; j++)
 		{
 			if(!garrisonSlots[j])
-				garrisonSlots[j] = std::make_shared<CGarrisonSlot>(this, i*garOffset.x + (j*distance), i*garOffset.y, SlotID(j), static_cast<CGarrisonSlot::EGarrisonType>(i), nullptr);
-			if(twoRows && j>=4)
+				garrisonSlots[j] = std::make_shared<CGarrisonSlot>(this, offset.x + (j*distance), offset.y, SlotID(j), i, nullptr);
+
+			if(layout == ESlotsLayout::TWO_ROWS && j >= 4)
 			{
 				garrisonSlots[j]->moveBy(Point(-126, 37));
 			}
+			else if(layout == ESlotsLayout::REVERSED_TWO_ROWS)
+			{
+				if(j >= 3)
+				{
+					garrisonSlots[j]->moveBy(Point(-90, 49));
+				}
+				else
+				{
+					garrisonSlots[j]->moveBy(Point(36, 0));
+				}
+			}
 		}
 		vstd::concatenate(availableSlots, garrisonSlots);
 	}
@@ -540,9 +590,10 @@ void CGarrisonInt::splitClick()
 	setSplittingMode(!getSplittingMode());
 	redraw();
 }
-void CGarrisonInt::splitStacks(int, int amountRight)
+
+void CGarrisonInt::splitStacks(const CGarrisonSlot * from, const CArmedInstance * armyDest, SlotID slotDest, int amount )
 {
-	LOCPLINT->cb->splitStack(armedObjs[getSelection()->upg], armedObjs[pb], getSelection()->ID, p2, amountRight);
+	LOCPLINT->cb->splitStack(armedObjs[from->upg], armyDest, from->ID, slotDest, amount);
 }
 
 bool CGarrisonInt::checkSelected(const CGarrisonSlot * selected, TQuantity min) const
@@ -556,9 +607,9 @@ void CGarrisonInt::moveStackToAnotherArmy(const CGarrisonSlot * selected)
 		return;
 
 	const auto srcArmyType = selected->upg;
-	const auto destArmyType = srcArmyType == CGarrisonSlot::UP
-		? CGarrisonSlot::DOWN
-		: CGarrisonSlot::UP;
+	const auto destArmyType = srcArmyType == EGarrisonType::UPPER
+		? EGarrisonType::LOWER
+		: EGarrisonType::UPPER;
 
 	auto srcArmy = armedObjs[srcArmyType];
 	auto destArmy = armedObjs[destArmyType];
@@ -600,9 +651,9 @@ void CGarrisonInt::bulkMoveArmy(const CGarrisonSlot * selected)
 		return;
 
 	const auto srcArmyType = selected->upg;
-	const auto destArmyType = (srcArmyType == CGarrisonSlot::UP)
-		? CGarrisonSlot::DOWN
-		: CGarrisonSlot::UP;
+	const auto destArmyType = (srcArmyType == EGarrisonType::UPPER)
+		? EGarrisonType::LOWER
+		: EGarrisonType::UPPER;
 
 	auto srcArmy = armedObjs[srcArmyType];
 	auto destArmy = armedObjs[destArmyType];
@@ -654,24 +705,20 @@ void CGarrisonInt::bulkSmartSplitStack(const CGarrisonSlot * selected)
 	LOCPLINT->cb->bulkSmartSplitStack(armedObjs[type]->id, selected->ID);
 }
 
-CGarrisonInt::CGarrisonInt(int x, int y, int inx, const Point & garsOffset,
-		const CArmedInstance * s1, const CArmedInstance * s2,
-		bool _removableUnits, bool smallImgs, bool _twoRows)
-	: highlighted(nullptr),
-    inSplittingMode(false),
-    interx(inx),
-    garOffset(garsOffset),
-    pb(false),
-    smallIcons(smallImgs),
-    removableUnits(_removableUnits),
-    twoRows(_twoRows)
+CGarrisonInt::CGarrisonInt(const Point & position, int inx, const Point & garsOffset, const CArmedInstance * s1, const CArmedInstance * s2, bool _removableUnits, bool smallImgs, ESlotsLayout _layout)
+	: highlighted(nullptr)
+	, inSplittingMode(false)
+	, interx(inx)
+	, garOffset(garsOffset)
+	, smallIcons(smallImgs)
+	, removableUnits(_removableUnits)
+	, layout(_layout)
 {
 	OBJECT_CONSTRUCTION_CAPTURING(255-DISPOSE);
 
-	setArmy(s1, false);
-	setArmy(s2, true);
-	pos.x += x;
-	pos.y += y;
+	setArmy(s1, EGarrisonType::UPPER);
+	setArmy(s2, EGarrisonType::LOWER);
+	pos += position;
 	createSlots();
 }
 
@@ -716,19 +763,51 @@ bool CGarrisonInt::getSplittingMode()
 	return inSplittingMode;
 }
 
-SlotID CGarrisonInt::getEmptySlot(CGarrisonSlot::EGarrisonType type) const
+SlotID CGarrisonInt::getEmptySlot(EGarrisonType type) const
 {
-	assert(armedObjs[type]);
-	return armedObjs[type] ? armedObjs[type]->getFreeSlot() : SlotID();
+	assert(army(type));
+	return army(type) ? army(type)->getFreeSlot() : SlotID();
 }
 
-bool CGarrisonInt::hasEmptySlot(CGarrisonSlot::EGarrisonType type) const
+bool CGarrisonInt::hasEmptySlot(EGarrisonType type) const
 {
 	return getEmptySlot(type) != SlotID();
 }
 
-void CGarrisonInt::setArmy(const CArmedInstance * army, bool bottomGarrison)
+const CArmedInstance * CGarrisonInt::upperArmy() const
 {
-	owned[bottomGarrison] =  army ? (army->tempOwner == LOCPLINT->playerID || army->tempOwner == PlayerColor::UNFLAGGABLE) : false;
-	armedObjs[bottomGarrison] = army;
+	return army(EGarrisonType::UPPER);
+}
+
+const CArmedInstance * CGarrisonInt::lowerArmy() const
+{
+	return army(EGarrisonType::LOWER);
+}
+
+const CArmedInstance * CGarrisonInt::army(EGarrisonType which) const
+{
+	if(armedObjs.count(which))
+		return armedObjs.at(which);
+	return nullptr;
+}
+
+bool CGarrisonInt::isArmyOwned(EGarrisonType which) const
+{
+	const auto * object = army(which);
+
+	if (!object)
+		return false;
+
+	if (object->tempOwner == LOCPLINT->playerID)
+		return true;
+
+	if (object->tempOwner == PlayerColor::UNFLAGGABLE)
+		return true;
+
+	return false;
+}
+
+void CGarrisonInt::setArmy(const CArmedInstance * army, EGarrisonType type)
+{
+	armedObjs[type] = army;
 }
