@@ -13,6 +13,22 @@
 VCMI_LIB_NAMESPACE_BEGIN
 
 class IBonusBearer;
+class FactionID;
+class TerrainId;
+
+class DLL_LINKAGE IConstBonusProvider
+{
+public:
+	virtual const IBonusBearer * getBonusBearer() const = 0;
+};
+
+class DLL_LINKAGE INativeTerrainProvider
+{
+public:
+	virtual TerrainId getNativeTerrain() const = 0;
+	virtual FactionID getFaction() const = 0;
+	virtual bool isNativeTerrain(TerrainId terrain) const;
+};
 
 class DLL_LINKAGE Entity
 {
@@ -23,8 +39,9 @@ public:
 
 	virtual int32_t getIndex() const = 0;
 	virtual int32_t getIconIndex() const = 0;
-	virtual const std::string & getJsonKey() const = 0;
-	virtual const std::string & getName() const = 0;
+	virtual std::string getJsonKey() const = 0;
+	virtual std::string getNameTranslated() const = 0;
+	virtual std::string getNameTextID() const = 0;
 
 	virtual void registerIcons(const IconRegistar & cb) const = 0;
 };
@@ -33,14 +50,14 @@ template <typename IdType>
 class DLL_LINKAGE EntityT : public Entity
 {
 public:
+	using IdentifierType = IdType;
+
 	virtual IdType getId() const = 0;
 };
 
 template <typename IdType>
-class DLL_LINKAGE EntityWithBonuses : public EntityT<IdType>
+class DLL_LINKAGE EntityWithBonuses : public EntityT<IdType>, public IConstBonusProvider
 {
-public:
-	virtual const IBonusBearer * accessBonuses() const = 0;
 };
 
 VCMI_LIB_NAMESPACE_END
