@@ -19,17 +19,24 @@ class Lobby : public QWidget
 {
 	Q_OBJECT
 
+	void changeEvent(QEvent *event) override;
 public:
 	explicit Lobby(QWidget *parent = nullptr);
 	~Lobby();
+	
+signals:
+	
+	void enableMod(QString mod);
+	void disableMod(QString mod);
+	
+public slots:
+	void updateMods();
 
 private slots:
-	void on_messageEdit_returnPressed();
-
-	void chatMessage(QString title, QString body, bool isSystem = false);
-	void sysMessage(QString body);
 	void dispatchMessage(QString);
 	void serverCommand(const ServerCommand &);
+	void onMessageSent(QString message);
+	void onChannelSwitch(QString channel);
 
 	void on_connectButton_toggled(bool checked);
 
@@ -49,9 +56,16 @@ private slots:
 
 	void on_kickButton_clicked();
 
+	void on_buttonResolve_clicked();
+
+	void on_optNewGame_toggled(bool checked);
+
+	void on_optLoadGame_toggled(bool checked);
+
 private:
 	QString serverUrl;
 	int serverPort;
+	bool isLoadGameMode = false;
 	
 	Ui::Lobby *ui;
 	SocketLobby socketLobby;
@@ -59,6 +73,8 @@ private:
 	QString session;
 	QString username;
 	QStringList gameArgs;
+	QMap<QString, QString> hostModsMap;
+	QMap<QString, QMap<QString, QString>> clientsModsMap;
 
 	enum AuthStatus
 	{
