@@ -35,20 +35,15 @@ class CModListView : public QWidget
 	CModFilterModel * filterModel;
 	CDownloadManager * dlManager;
 
-	SettingsListener settingsListener;
-	bool repositoriesChanged;
-
-	void showEvent(QShowEvent * event) override;
-
-	void keyPressEvent(QKeyEvent * event) override;
-
 	void setupModModel();
 	void setupFilterModel();
 	void setupModsView();
-	void loadRepositories();
 
 	void checkManagerErrors();
 
+	/// replace mod ID's with proper human-readable mod names
+	QStringList getModNames(QStringList input);
+
 	// find mods unknown to mod list (not present in repo and not installed)
 	QStringList findInvalidDependencies(QString mod);
 	// find mods that block enabling of this mod: conflicting with this mod or one of required mods
@@ -56,7 +51,7 @@ class CModListView : public QWidget
 	// find mods that depend on this one
 	QStringList findDependentMods(QString mod, bool excludeDisabled);
 
-	void downloadFile(QString file, QString url, QString description);
+	void downloadFile(QString file, QString url, QString description, qint64 size = 0);
 
 	void installMods(QStringList archives);
 	void installFiles(QStringList mods);
@@ -64,24 +59,40 @@ class CModListView : public QWidget
 	QString genChangelogText(CModEntry & mod);
 	QString genModInfoText(CModEntry & mod);
 
+	void changeEvent(QEvent *event) override;
 signals:
-	void extraResolutionsEnabledChanged(bool enabled);
+	void modsChanged();
 
 public:
-	explicit CModListView(QWidget * parent = 0);
+	explicit CModListView(QWidget * parent = nullptr);
 	~CModListView();
 
-	void showModInfo();
-	void hideModInfo();
 	void loadScreenshots();
+	void loadRepositories();
 
-	void enableModInfo();
 	void disableModInfo();
 
 	void selectMod(const QModelIndex & index);
-	bool isExtraResolutionsModEnabled() const;
 
 	const CModList & getModList() const;
+	
+	// First Launch View interface
+
+	/// install mod by name
+	void doInstallMod(const QString & modName);
+
+	/// returns true if mod is available in repository and can be installed
+	bool isModAvailable(const QString & modName);
+
+	/// finds translation mod for specified languages. Returns empty string on error
+	QString getTranslationModName(const QString & language);
+
+	/// returns true if mod is currently enabled
+	bool isModEnabled(const QString & modName);
+
+public slots:
+	void enableModByName(QString modName);
+	void disableModByName(QString modName);
 
 private slots:
 	void dataChanged(const QModelIndex & topleft, const QModelIndex & bottomRight);
@@ -91,8 +102,6 @@ private slots:
 	void modelReset();
 	void hideProgressBar();
 
-	void on_hideModInfoButton_clicked();
-
 	void on_lineEdit_textChanged(const QString & arg1);
 
 	void on_comboBox_currentIndexChanged(int index);
@@ -117,7 +126,7 @@ private slots:
 
 	void on_screenshotsList_clicked(const QModelIndex & index);
 
-	void on_showInfoButton_clicked();
+	void on_allModsView_doubleClicked(const QModelIndex &index);
 
 private:
 	Ui::CModListView * ui;
