@@ -20,6 +20,15 @@ ImageViewer::ImageViewer(QWidget * parent)
 	ui->setupUi(this);
 }
 
+void ImageViewer::changeEvent(QEvent *event)
+{
+	if(event->type() == QEvent::LanguageChange)
+	{
+		ui->retranslateUi(this);
+	}
+	QDialog::changeEvent(event);
+}
+
 ImageViewer::~ImageViewer()
 {
 	delete ui;
