@@ -10,18 +10,22 @@
 #include "StdInc.h"
 #include "CCreatureSet.h"
 
+#include "ArtifactUtils.h"
+#include "CConfigHandler.h"
 #include "CCreatureHandler.h"
 #include "VCMI_Lib.h"
-#include "CModHandler.h"
+#include "GameSettings.h"
 #include "mapObjects/CGHeroInstance.h"
+#include "modding/ModScope.h"
 #include "IGameCallback.h"
-#include "CGameState.h"
 #include "CGeneralTextHandler.h"
 #include "spells/CSpellHandler.h"
 #include "CHeroHandler.h"
 #include "IBonusTypeHandler.h"
 #include "serializer/JsonSerializeFormat.h"
-#include "NetPacksBase.h"
+
+#include <vcmi/FactionService.h>
+#include <vcmi/Faction.h>
 
 VCMI_LIB_NAMESPACE_BEGIN
 
@@ -31,7 +35,7 @@ bool CreatureSlotComparer::operator()(const TPairCreatureSlot & lhs, const TPair
 	return lhs.first->getAIValue() < rhs.first->getAIValue(); // Descendant order sorting
 }
 
-const CStackInstance &CCreatureSet::operator[](SlotID slot) const
+const CStackInstance & CCreatureSet::operator[](const SlotID & slot) const
 {
 	auto i = stacks.find(slot);
 	if (i != stacks.end())
@@ -40,7 +44,7 @@ const CStackInstance &CCreatureSet::operator[](SlotID slot) const
 		throw std::runtime_error("That slot is empty!");
 }
 
-const CCreature* CCreatureSet::getCreature(SlotID slot) const
+const CCreature * CCreatureSet::getCreature(const SlotID & slot) const
 {
 	auto i = stacks.find(slot);
 	if (i != stacks.end())
@@ -66,22 +70,22 @@ bool CCreatureSet::setCreature(SlotID slot, CreatureID type, TQuantity quantity)
 	if(hasStackAtSlot(slot)) //remove old creature
 		eraseStack(slot);
 
-	auto armyObj = castToArmyObj();
+	auto * armyObj = castToArmyObj();
 	bool isHypotheticArmy = armyObj ? armyObj->isHypothetic() : false;
 
 	putStack(slot, new CStackInstance(type, quantity, isHypotheticArmy));
 	return true;
 }
 
-SlotID CCreatureSet::getSlotFor(CreatureID creature, ui32 slotsAmount) const /*returns -1 if no slot available */
+SlotID CCreatureSet::getSlotFor(const CreatureID & creature, ui32 slotsAmount) const /*returns -1 if no slot available */
 {
-	return getSlotFor(VLC->creh->objects[creature], slotsAmount);
+	return getSlotFor(creature.toCreature(), slotsAmount);
 }
 
 SlotID CCreatureSet::getSlotFor(const CCreature *c, ui32 slotsAmount) const
 {
 	assert(c && c->valid());
-	for(auto & elem : stacks)
+	for(const auto & elem : stacks)
 	{
 		assert(elem.second->type->valid());
 		if(elem.second->type == c)
@@ -92,10 +96,10 @@ SlotID CCreatureSet::getSlotFor(const CCreature *c, ui32 slotsAmount) const
 	return getFreeSlot(slotsAmount);
 }
 
-bool CCreatureSet::hasCreatureSlots(const CCreature * c, SlotID exclude) const
+bool CCreatureSet::hasCreatureSlots(const CCreature * c, const SlotID & exclude) const
 {
 	assert(c && c->valid());
-	for(auto & elem : stacks) // elem is const
+	for(const auto & elem : stacks) // elem is const
 	{
 		if(elem.first == exclude) // Check slot
 			continue;
@@ -111,12 +115,12 @@ bool CCreatureSet::hasCreatureSlots(const CCreature * c, SlotID exclude) const
 	return false;
 }
 
-std::vector<SlotID> CCreatureSet::getCreatureSlots(const CCreature * c, SlotID exclude, TQuantity ignoreAmount) const
+std::vector<SlotID> CCreatureSet::getCreatureSlots(const CCreature * c, const SlotID & exclude, TQuantity ignoreAmount) const
 {
 	assert(c && c->valid());
 	std::vector<SlotID> result;
 
-	for(auto & elem : stacks)
+	for(const auto & elem : stacks)
 	{
 		if(elem.first == exclude)
 			continue;
@@ -139,7 +143,7 @@ bool CCreatureSet::isCreatureBalanced(const CCreature * c, TQuantity ignoreAmoun
 	TQuantity max = 0;
 	TQuantity min = std::numeric_limits<TQuantity>::max();
 
-	for(auto & elem : stacks)
+	for(const auto & elem : stacks)
 	{
 		if(!elem.second || !elem.second->type || elem.second->type != c)
 			continue;
@@ -208,11 +212,11 @@ TMapCreatureSlot CCreatureSet::getCreatureMap() const
 
 	// https://stackoverflow.com/questions/97050/stdmap-insert-or-stdmap-find
 	// https://www.cplusplus.com/reference/map/map/key_comp/
-	for(auto pair : stacks)
+	for(const auto & pair : stacks)
 	{
-		auto creature = pair.second->type;
+		const auto * creature = pair.second->type;
 		auto slot = pair.first;
-		TMapCreatureSlot::iterator lb = creatureMap.lower_bound(creature);
+		auto lb = creatureMap.lower_bound(creature);
 
 		if(lb != creatureMap.end() && !(keyComp(creature, lb->first)))
 			continue;
@@ -222,11 +226,11 @@ TMapCreatureSlot CCreatureSet::getCreatureMap() const
 	return creatureMap;
 }
 
-TCreatureQueue CCreatureSet::getCreatureQueue(SlotID exclude) const
+TCreatureQueue CCreatureSet::getCreatureQueue(const SlotID & exclude) const
 {
 	TCreatureQueue creatureQueue;
 
-	for(auto pair : stacks)
+	for(const auto & pair : stacks)
 	{
 		if(pair.first == exclude)
 			continue;
@@ -235,7 +239,7 @@ TCreatureQueue CCreatureSet::getCreatureQueue(SlotID exclude) const
 	return creatureQueue;
 }
 
-TQuantity CCreatureSet::getStackCount(SlotID slot) const
+TQuantity CCreatureSet::getStackCount(const SlotID & slot) const
 {
 	auto i = stacks.find(slot);
 	if (i != stacks.end())
@@ -244,7 +248,7 @@ TQuantity CCreatureSet::getStackCount(SlotID slot) const
 		return 0; //TODO? consider issuing a warning
 }
 
-TExpType CCreatureSet::getStackExperience(SlotID slot) const
+TExpType CCreatureSet::getStackExperience(const SlotID & slot) const
 {
 	auto i = stacks.find(slot);
 	if (i != stacks.end())
@@ -253,14 +257,13 @@ TExpType CCreatureSet::getStackExperience(SlotID slot) const
 		return 0; //TODO? consider issuing a warning
 }
 
-
-bool CCreatureSet::mergableStacks(std::pair<SlotID, SlotID> &out, SlotID preferable) const /*looks for two same stacks, returns slot positions */
+bool CCreatureSet::mergableStacks(std::pair<SlotID, SlotID> & out, const SlotID & preferable) const /*looks for two same stacks, returns slot positions */
 {
 	//try to match creature to our preferred stack
 	if(preferable.validSlot() &&  vstd::contains(stacks, preferable))
 	{
 		const CCreature *cr = stacks.find(preferable)->second->type;
-		for(auto & elem : stacks)
+		for(const auto & elem : stacks)
 		{
 			if(cr == elem.second->type && elem.first != preferable)
 			{
@@ -271,13 +274,13 @@ bool CCreatureSet::mergableStacks(std::pair<SlotID, SlotID> &out, SlotID prefera
 		}
 	}
 
-	for(auto i=stacks.begin(); i!=stacks.end(); ++i)
+	for(const auto & stack : stacks)
 	{
-		for(auto & elem : stacks)
+		for(const auto & elem : stacks)
 		{
-			if(i->second->type == elem.second->type  &&  i->first != elem.first)
+			if(stack.second->type == elem.second->type && stack.first != elem.first)
 			{
-				out.first = i->first;
+				out.first = stack.first;
 				out.second = elem.first;
 				return true;
 			}
@@ -299,9 +302,9 @@ void CCreatureSet::sweep()
 	}
 }
 
-void CCreatureSet::addToSlot(SlotID slot, CreatureID cre, TQuantity count, bool allowMerging)
+void CCreatureSet::addToSlot(const SlotID & slot, const CreatureID & cre, TQuantity count, bool allowMerging)
 {
-	const CCreature *c = VLC->creh->objects[cre];
+	const CCreature *c = cre.toCreature();
 
 	if(!hasStackAtSlot(slot))
 	{
@@ -317,7 +320,7 @@ void CCreatureSet::addToSlot(SlotID slot, CreatureID cre, TQuantity count, bool
 	}
 }
 
-void CCreatureSet::addToSlot(SlotID slot, CStackInstance *stack, bool allowMerging)
+void CCreatureSet::addToSlot(const SlotID & slot, CStackInstance * stack, bool allowMerging)
 {
 	assert(stack->valid(true));
 
@@ -337,7 +340,7 @@ void CCreatureSet::addToSlot(SlotID slot, CStackInstance *stack, bool allowMergi
 
 bool CCreatureSet::validTypes(bool allowUnrandomized) const
 {
-	for(auto & elem : stacks)
+	for(const auto & elem : stacks)
 	{
 		if(!elem.second->valid(allowUnrandomized))
 			return false;
@@ -345,7 +348,7 @@ bool CCreatureSet::validTypes(bool allowUnrandomized) const
 	return true;
 }
 
-bool CCreatureSet::slotEmpty(SlotID slot) const
+bool CCreatureSet::slotEmpty(const SlotID & slot) const
 {
 	return !hasStackAtSlot(slot);
 }
@@ -358,23 +361,29 @@ bool CCreatureSet::needsLastStack() const
 ui64 CCreatureSet::getArmyStrength() const
 {
 	ui64 ret = 0;
-	for(auto & elem : stacks)
+	for(const auto & elem : stacks)
 		ret += elem.second->getPower();
 	return ret;
 }
 
-ui64 CCreatureSet::getPower (SlotID slot) const
+ui64 CCreatureSet::getPower(const SlotID & slot) const
 {
 	return getStack(slot).getPower();
 }
 
-std::string CCreatureSet::getRoughAmount(SlotID slot, int mode) const
+std::string CCreatureSet::getRoughAmount(const SlotID & slot, int mode) const
 {
 	/// Mode represent return string format
 	/// "Pack" - 0, "A pack of" - 1, "a pack of" - 2
-	int quantity = CCreature::getQuantityID(getStackCount(slot));
-	if(quantity)
-		return VLC->generaltexth->arraytxt[(174 + mode) + 3*CCreature::getQuantityID(getStackCount(slot))];
+	CCreature::CreatureQuantityId quantity = CCreature::getQuantityID(getStackCount(slot));
+
+	if((int)quantity)
+	{
+		if(settings["gameTweaks"]["numericCreaturesQuantities"].Bool())
+			return CCreature::getQuantityRangeStringForId(quantity);
+
+		return VLC->generaltexth->arraytxt[(174 + mode) + 3*(int)quantity];
+	}
 	return "";
 }
 
@@ -382,12 +391,12 @@ std::string CCreatureSet::getArmyDescription() const
 {
 	std::string text;
 	std::vector<std::string> guards;
-	for(auto & elem : stacks)
+	for(const auto & elem : stacks)
 	{
-		auto str = boost::str(boost::format("%s %s") % getRoughAmount(elem.first, 2) % getCreature(elem.first)->namePl);
+		auto str = boost::str(boost::format("%s %s") % getRoughAmount(elem.first, 2) % getCreature(elem.first)->getNamePluralTranslated());
 		guards.push_back(str);
 	}
-	if(guards.size())
+	if(!guards.empty())
 	{
 		for(int i = 0; i < guards.size(); i++)
 		{
@@ -406,16 +415,16 @@ int CCreatureSet::stacksCount() const
 	return static_cast<int>(stacks.size());
 }
 
-void CCreatureSet::setFormation(bool tight)
+void CCreatureSet::setFormation(EArmyFormation mode)
 {
-	formation = tight;
+	formation = mode;
 }
 
-void CCreatureSet::setStackCount(SlotID slot, TQuantity count)
+void CCreatureSet::setStackCount(const SlotID & slot, TQuantity count)
 {
 	assert(hasStackAtSlot(slot));
 	assert(stacks[slot]->count + count > 0);
-	if (VLC->modh->modules.STACK_EXP && count > stacks[slot]->count)
+	if (VLC->settings()->getBoolean(EGameSettings::MODULE_STACK_EXPERIENCE) && count > stacks[slot]->count)
 		stacks[slot]->experience = static_cast<TExpType>(stacks[slot]->experience * (count / static_cast<double>(stacks[slot]->count)));
 	stacks[slot]->count = count;
 	armyChanged();
@@ -426,13 +435,13 @@ void CCreatureSet::giveStackExp(TExpType exp)
 	for(TSlots::const_iterator i = stacks.begin(); i != stacks.end(); i++)
 		i->second->giveStackExp(exp);
 }
-void CCreatureSet::setStackExp(SlotID slot, TExpType exp)
+void CCreatureSet::setStackExp(const SlotID & slot, TExpType exp)
 {
 	assert(hasStackAtSlot(slot));
 	stacks[slot]->experience = exp;
 }
 
-void CCreatureSet::clear()
+void CCreatureSet::clearSlots()
 {
 	while(!stacks.empty())
 	{
@@ -440,20 +449,20 @@ void CCreatureSet::clear()
 	}
 }
 
-const CStackInstance& CCreatureSet::getStack(SlotID slot) const
+const CStackInstance & CCreatureSet::getStack(const SlotID & slot) const
 {
 	assert(hasStackAtSlot(slot));
 	return *getStackPtr(slot);
 }
 
-const CStackInstance* CCreatureSet::getStackPtr(SlotID slot) const
+CStackInstance * CCreatureSet::getStackPtr(const SlotID & slot) const
 {
 	if(hasStackAtSlot(slot))
 		return stacks.find(slot)->second;
 	else return nullptr;
 }
 
-void CCreatureSet::eraseStack(SlotID slot)
+void CCreatureSet::eraseStack(const SlotID & slot)
 {
 	assert(hasStackAtSlot(slot));
 	CStackInstance *toErase = detachStack(slot);
@@ -465,7 +474,7 @@ bool CCreatureSet::contains(const CStackInstance *stack) const
 	if(!stack)
 		return false;
 
-	for(auto & elem : stacks)
+	for(const auto & elem : stacks)
 		if(elem.second == stack)
 			return true;
 
@@ -474,14 +483,14 @@ bool CCreatureSet::contains(const CStackInstance *stack) const
 
 SlotID CCreatureSet::findStack(const CStackInstance *stack) const
 {
-	auto h = dynamic_cast<const CGHeroInstance *>(this);
+	const auto * h = dynamic_cast<const CGHeroInstance *>(this);
 	if (h && h->commander == stack)
 		return SlotID::COMMANDER_SLOT_PLACEHOLDER;
 
 	if(!stack)
 		return SlotID();
 
-	for(auto & elem : stacks)
+	for(const auto & elem : stacks)
 		if(elem.second == stack)
 			return elem.first;
 
@@ -493,7 +502,7 @@ CArmedInstance * CCreatureSet::castToArmyObj()
 	return dynamic_cast<CArmedInstance *>(this);
 }
 
-void CCreatureSet::putStack(SlotID slot, CStackInstance *stack)
+void CCreatureSet::putStack(const SlotID & slot, CStackInstance * stack)
 {
 	assert(slot.getNum() < GameConstants::ARMY_SIZE);
 	assert(!hasStackAtSlot(slot));
@@ -502,41 +511,30 @@ void CCreatureSet::putStack(SlotID slot, CStackInstance *stack)
 	armyChanged();
 }
 
-void CCreatureSet::joinStack(SlotID slot, CStackInstance * stack)
+void CCreatureSet::joinStack(const SlotID & slot, CStackInstance * stack)
 {
-	const CCreature *c = getCreature(slot);
+	[[maybe_unused]] const CCreature *c = getCreature(slot);
 	assert(c == stack->type);
 	assert(c);
-	UNUSED(c);
 
 	//TODO move stuff
 	changeStackCount(slot, stack->count);
 	vstd::clear_pointer(stack);
 }
 
-void CCreatureSet::changeStackCount(SlotID slot, TQuantity toAdd)
+void CCreatureSet::changeStackCount(const SlotID & slot, TQuantity toAdd)
 {
 	setStackCount(slot, getStackCount(slot) + toAdd);
 }
 
-CCreatureSet::CCreatureSet()
-{
-	formation = false;
-}
-
-CCreatureSet::CCreatureSet(const CCreatureSet&)
-{
-	assert(0);
-}
-
 CCreatureSet::~CCreatureSet()
 {
-	clear();
+	clearSlots();
 }
 
 void CCreatureSet::setToArmy(CSimpleArmy &src)
 {
-	clear();
+	clearSlots();
 	while(src)
 	{
 		auto i = src.army.begin();
@@ -546,7 +544,7 @@ void CCreatureSet::setToArmy(CSimpleArmy &src)
 	}
 }
 
-CStackInstance * CCreatureSet::detachStack(SlotID slot)
+CStackInstance * CCreatureSet::detachStack(const SlotID & slot)
 {
 	assert(hasStackAtSlot(slot));
 	CStackInstance *ret = stacks[slot];
@@ -563,7 +561,7 @@ CStackInstance * CCreatureSet::detachStack(SlotID slot)
 	return ret;
 }
 
-void CCreatureSet::setStackType(SlotID slot, CreatureID type)
+void CCreatureSet::setStackType(const SlotID & slot, const CreatureID & type)
 {
 	assert(hasStackAtSlot(slot));
 	CStackInstance *s = stacks[slot];
@@ -577,7 +575,7 @@ bool CCreatureSet::canBeMergedWith(const CCreatureSet &cs, bool allowMergingStac
 	{
 		int freeSlots = stacksCount() - GameConstants::ARMY_SIZE;
 		std::set<const CCreature*> cresToAdd;
-		for(auto & elem : cs.stacks)
+		for(const auto & elem : cs.stacks)
 		{
 			SlotID dest = getSlotFor(elem.second->type);
 			if(!dest.validSlot() || hasStackAtSlot(dest))
@@ -591,14 +589,14 @@ bool CCreatureSet::canBeMergedWith(const CCreatureSet &cs, bool allowMergingStac
 		SlotID j;
 
 		//get types of creatures that need their own slot
-		for(auto & elem : cs.stacks)
+		for(const auto & elem : cs.stacks)
 			if ((j = cres.getSlotFor(elem.second->type)).validSlot())
-				cres.addToSlot(j, elem.second->type->idNumber, 1, true);  //merge if possible
-			//cres.addToSlot(elem.first, elem.second->type->idNumber, 1, true);
-		for(auto & elem : stacks)
+				cres.addToSlot(j, elem.second->type->getId(), 1, true);  //merge if possible
+			//cres.addToSlot(elem.first, elem.second->type->getId(), 1, true);
+		for(const auto & elem : stacks)
 		{
 			if ((j = cres.getSlotFor(elem.second->type)).validSlot())
-				cres.addToSlot(j, elem.second->type->idNumber, 1, true);  //merge if possible
+				cres.addToSlot(j, elem.second->type->getId(), 1, true);  //merge if possible
 			else
 				return false; //no place found
 		}
@@ -606,7 +604,7 @@ bool CCreatureSet::canBeMergedWith(const CCreatureSet &cs, bool allowMergingStac
 	}
 }
 
-bool CCreatureSet::hasStackAtSlot(SlotID slot) const
+bool CCreatureSet::hasStackAtSlot(const SlotID & slot) const
 {
 	return vstd::contains(stacks, slot);
 }
@@ -622,12 +620,13 @@ void CCreatureSet::armyChanged()
 
 }
 
-void CCreatureSet::serializeJson(JsonSerializeFormat & handler, const std::string & fieldName, const boost::optional<int> fixedSize)
+void CCreatureSet::serializeJson(JsonSerializeFormat & handler, const std::string & armyFieldName, const std::optional<int> fixedSize)
 {
 	if(handler.saving && stacks.empty())
 		return;
 
-	auto a = handler.enterArray(fieldName);
+	handler.serializeEnum("formation", formation, NArmyFormation::names);
+	auto a = handler.enterArray(armyFieldName);
 
 
 	if(handler.saving)
@@ -638,7 +637,7 @@ void CCreatureSet::serializeJson(JsonSerializeFormat & handler, const std::strin
 			vstd::amax(sz, p.first.getNum()+1);
 
 		if(fixedSize)
-			vstd::amax(sz, fixedSize.get());
+			vstd::amax(sz, fixedSize.value());
 
 		a.resize(sz, JsonNode::JsonType::DATA_STRUCT);
 
@@ -660,9 +659,9 @@ void CCreatureSet::serializeJson(JsonSerializeFormat & handler, const std::strin
 
 			if(amount > 0)
 			{
-				CStackInstance * new_stack = new CStackInstance();
+				auto * new_stack = new CStackInstance();
 				new_stack->serializeJson(handler);
-				putStack(SlotID((si32)idx), new_stack);
+				putStack(SlotID(static_cast<si32>(idx)), new_stack);
 			}
 		}
 	}
@@ -674,8 +673,8 @@ CStackInstance::CStackInstance()
 	init();
 }
 
-CStackInstance::CStackInstance(CreatureID id, TQuantity Count, bool isHypothetic)
-	: CBonusSystemNode(isHypothetic), armyObj(_armyObj)
+CStackInstance::CStackInstance(const CreatureID & id, TQuantity Count, bool isHypothetic):
+	CBonusSystemNode(isHypothetic), armyObj(_armyObj)
 {
 	init();
 	setType(id);
@@ -695,24 +694,23 @@ void CStackInstance::init()
 	experience = 0;
 	count = 0;
 	type = nullptr;
-	idRand = -1;
 	_armyObj = nullptr;
 	setNodeType(STACK_INSTANCE);
 }
 
-int CStackInstance::getQuantityID() const
+CCreature::CreatureQuantityId CStackInstance::getQuantityID() const
 {
 	return CCreature::getQuantityID(count);
 }
 
 int CStackInstance::getExpRank() const
 {
-	if (!VLC->modh->modules.STACK_EXP)
+	if (!VLC->settings()->getBoolean(EGameSettings::MODULE_STACK_EXPERIENCE))
 		return 0;
-	int tier = type->level;
+	int tier = type->getLevel();
 	if (vstd::iswithin(tier, 1, 7))
 	{
-		for (int i = (int)VLC->creh->expRanks[tier].size()-2; i >-1; --i)//sic!
+		for(int i = static_cast<int>(VLC->creh->expRanks[tier].size()) - 2; i > -1; --i) //sic!
 		{ //exp values vary from 1st level to max exp at 11th level
 			if (experience >= VLC->creh->expRanks[tier][i])
 				return ++i; //faster, but confusing - 0 index mean 1st level of experience
@@ -721,7 +719,7 @@ int CStackInstance::getExpRank() const
 	}
 	else //higher tier
 	{
-		for (int i = (int)VLC->creh->expRanks[0].size()-2; i >-1; --i)
+		for(int i = static_cast<int>(VLC->creh->expRanks[0].size()) - 2; i > -1; --i)
 		{
 			if (experience >= VLC->creh->expRanks[0][i])
 				return ++i;
@@ -732,41 +730,29 @@ int CStackInstance::getExpRank() const
 
 int CStackInstance::getLevel() const
 {
-	return std::max (1, (int)type->level);
-}
-
-si32 CStackInstance::magicResistance() const
-{
-	si32 val = valOfBonuses(Selector::type()(Bonus::MAGIC_RESISTANCE));
-	if (const CGHeroInstance * hero = dynamic_cast<const CGHeroInstance *>(_armyObj))
-	{
-		//resistance skill
-		val += hero->valOfBonuses(Bonus::SECONDARY_SKILL_PREMY, SecondarySkill::RESISTANCE);
-	}
-	vstd::amin (val, 100);
-	return val;
+	return std::max(1, static_cast<int>(type->getLevel()));
 }
 
 void CStackInstance::giveStackExp(TExpType exp)
 {
-	int level = type->level;
+	int level = type->getLevel();
 	if (!vstd::iswithin(level, 1, 7))
 		level = 0;
 
 	CCreatureHandler * creh = VLC->creh;
 	ui32 maxExp = creh->expRanks[level].back();
 
-	vstd::amin(exp, (TExpType)maxExp); //prevent exp overflow due to different types
+	vstd::amin(exp, static_cast<TExpType>(maxExp)); //prevent exp overflow due to different types
 	vstd::amin(exp, (maxExp * creh->maxExpPerBattle[level])/100);
 	vstd::amin(experience += exp, maxExp); //can't get more exp than this limit
 }
 
-void CStackInstance::setType(CreatureID creID)
+void CStackInstance::setType(const CreatureID & creID)
 {
-	if(creID >= 0 && creID < VLC->creh->objects.size())
-		setType(VLC->creh->objects[creID]);
+	if (creID == CreatureID::NONE)
+		setType(nullptr);//FIXME: unused branch?
 	else
-		setType((const CCreature*)nullptr);
+		setType(creID.toCreature());
 }
 
 void CStackInstance::setType(const CCreature *c)
@@ -774,7 +760,7 @@ void CStackInstance::setType(const CCreature *c)
 	if(type)
 	{
 		detachFrom(const_cast<CCreature&>(*type));
-		if (type->isMyUpgrade(c) && VLC->modh->modules.STACK_EXP)
+		if (type->isMyUpgrade(c) && VLC->settings()->getBoolean(EGameSettings::MODULE_STACK_EXPERIENCE))
 			experience = static_cast<TExpType>(experience * VLC->creh->expAfterUpgrade / 100.0);
 	}
 
@@ -785,18 +771,10 @@ void CStackInstance::setType(const CCreature *c)
 }
 std::string CStackInstance::bonusToString(const std::shared_ptr<Bonus>& bonus, bool description) const
 {
-	if(Bonus::MAGIC_RESISTANCE == bonus->type)
-	{
-		return "";
-	}
-	else
-	{
-		return VLC->getBth()->bonusToString(bonus, this, description);
-	}
-
+	return VLC->getBth()->bonusToString(bonus, this, description);
 }
 
-std::string CStackInstance::bonusToGraphics(const std::shared_ptr<Bonus>& bonus) const
+ImagePath CStackInstance::bonusToGraphics(const std::shared_ptr<Bonus> & bonus) const
 {
 	return VLC->getBth()->bonusToGraphics(bonus);
 }
@@ -814,38 +792,35 @@ void CStackInstance::setArmyObj(const CArmedInstance * ArmyObj)
 
 std::string CStackInstance::getQuantityTXT(bool capitalized) const
 {
-	int quantity = getQuantityID();
+	CCreature::CreatureQuantityId quantity = getQuantityID();
 
-	if (quantity)
-		return VLC->generaltexth->arraytxt[174 + quantity*3 - 1 - capitalized];
+	if ((int)quantity)
+	{
+		if(settings["gameTweaks"]["numericCreaturesQuantities"].Bool())
+			return CCreature::getQuantityRangeStringForId(quantity);
+
+		return VLC->generaltexth->arraytxt[174 + (int)quantity*3 - 1 - capitalized];
+	}
 	else
 		return "";
 }
 
 bool CStackInstance::valid(bool allowUnrandomized) const
 {
-	bool isRand = (idRand != -1);
-	if(!isRand)
+	if(!randomStack)
 	{
-		return (type  &&  type == VLC->creh->objects[type->idNumber]);
+		return (type && type == type->getId().toEntity(VLC));
 	}
 	else
 		return allowUnrandomized;
 }
 
-CStackInstance::~CStackInstance()
-{
-
-}
-
 std::string CStackInstance::nodeName() const
 {
 	std::ostringstream oss;
 	oss << "Stack of " << count << " of ";
 	if(type)
-		oss << type->namePl;
-	else if(idRand >= 0)
-		oss << "[no type, idRand=" << idRand << "]";
+		oss << type->getNamePluralTextID();
 	else
 		oss << "[UNDEFINED TYPE]";
 
@@ -868,20 +843,20 @@ void CStackInstance::deserializationFix()
 CreatureID CStackInstance::getCreatureID() const
 {
 	if(type)
-		return type->idNumber;
+		return type->getId();
 	else
 		return CreatureID::NONE;
 }
 
 std::string CStackInstance::getName() const
 {
-	return (count > 1) ? type->namePl : type->nameSing;
+	return (count > 1) ? type->getNamePluralTranslated() : type->getNameSingularTranslated();
 }
 
 ui64 CStackInstance::getPower() const
 {
 	assert(type);
-	return type->AIValue * count;
+	return type->getAIValue() * count;
 }
 
 ArtBearer::ArtBearer CStackInstance::bearerType() const
@@ -889,10 +864,21 @@ ArtBearer::ArtBearer CStackInstance::bearerType() const
 	return ArtBearer::CREATURE;
 }
 
-void CStackInstance::putArtifact(ArtifactPosition pos, CArtifactInstance * art)
+CStackInstance::ArtPlacementMap CStackInstance::putArtifact(ArtifactPosition pos, CArtifactInstance * art)
 {
 	assert(!getArt(pos));
-	art->putAt(ArtifactLocation(this, pos));
+	assert(art->canBePutAt(this, pos));
+
+	attachTo(*art);
+	return CArtifactSet::putArtifact(pos, art);
+}
+
+void CStackInstance::removeArtifact(ArtifactPosition pos)
+{
+	assert(getArt(pos));
+
+	detachFrom(*getArt(pos));
+	CArtifactSet::removeArtifact(pos);
 }
 
 void CStackInstance::serializeJson(JsonSerializeFormat & handler)
@@ -902,14 +888,13 @@ void CStackInstance::serializeJson(JsonSerializeFormat & handler)
 
 	if(handler.saving)
 	{
-		if(idRand > -1)
+		if(randomStack)
 		{
-			int level = (int)idRand / 2;
-
-			boost::logic::tribool upgraded = (idRand % 2) > 0;
+			int level = randomStack->level;
+			int upgrade = randomStack->upgrade;
 
 			handler.serializeInt("level", level, 0);
-			handler.serializeBool("upgraded", upgraded);
+			handler.serializeInt("upgraded", upgrade, 0);
 		}
 	}
 	else
@@ -917,28 +902,40 @@ void CStackInstance::serializeJson(JsonSerializeFormat & handler)
 		//type set by CStackBasicDescriptor::serializeJson
 		if(type == nullptr)
 		{
-			int level = 0;
-			bool upgraded = false;
+			uint8_t level = 0;
+			uint8_t upgrade = 0;
 
 			handler.serializeInt("level", level, 0);
-			handler.serializeBool("upgraded", upgraded);
+			handler.serializeInt("upgrade", upgrade, 0);
 
-			idRand = level * 2 + (int)(bool)upgraded;
+			randomStack = RandomStackInfo{ level, upgrade };
 		}
 	}
 }
 
+FactionID CStackInstance::getFaction() const
+{
+	if(type)
+		return type->getFaction();
+		
+	return FactionID::NEUTRAL;
+}
+
+const IBonusBearer* CStackInstance::getBonusBearer() const
+{
+	return this;
+}
+
 CCommanderInstance::CCommanderInstance()
 {
 	init();
-	name = "Unnamed";
 }
 
-CCommanderInstance::CCommanderInstance (CreatureID id)
+CCommanderInstance::CCommanderInstance(const CreatureID & id): name("Commando")
 {
 	init();
 	setType(id);
-	name = "Commando"; //TODO - parse them
+	//TODO - parse them
 }
 
 void CCommanderInstance::init()
@@ -948,17 +945,11 @@ void CCommanderInstance::init()
 	level = 1;
 	count = 1;
 	type = nullptr;
-	idRand = -1;
 	_armyObj = nullptr;
 	setNodeType (CBonusSystemNode::COMMANDER);
 	secondarySkills.resize (ECommander::SPELL_POWER + 1);
 }
 
-CCommanderInstance::~CCommanderInstance()
-{
-
-}
-
 void CCommanderInstance::setAlive (bool Alive)
 {
 	//TODO: helm of immortality
@@ -988,7 +979,7 @@ int CCommanderInstance::getLevel() const
 void CCommanderInstance::levelUp ()
 {
 	level++;
-	for (auto bonus : VLC->creh->commanderLevelPremy)
+	for(const auto & bonus : VLC->creh->commanderLevelPremy)
 	{ //grant all regular level-up bonuses
 		accumulateBonus(bonus);
 	}
@@ -1001,17 +992,15 @@ ArtBearer::ArtBearer CCommanderInstance::bearerType() const
 
 bool CCommanderInstance::gainsLevel() const
 {
-	return experience >= (TExpType)VLC->heroh->reqExp(level+1);
+	return experience >= static_cast<TExpType>(VLC->heroh->reqExp(level + 1));
 }
 
-CStackBasicDescriptor::CStackBasicDescriptor()
-{
-	type = nullptr;
-	count = -1;
-}
+//This constructor should be placed here to avoid side effects
+CStackBasicDescriptor::CStackBasicDescriptor() = default;
 
-CStackBasicDescriptor::CStackBasicDescriptor(CreatureID id, TQuantity Count)
-	: type (VLC->creh->objects[id]), count(Count)
+CStackBasicDescriptor::CStackBasicDescriptor(const CreatureID & id, TQuantity Count):
+	type(id.toCreature()),
+	count(Count)
 {
 }
 
@@ -1025,6 +1014,11 @@ const Creature * CStackBasicDescriptor::getType() const
 	return type;
 }
 
+CreatureID CStackBasicDescriptor::getId() const
+{
+	return type->getId();
+}
+
 TQuantity CStackBasicDescriptor::getCount() const
 {
 	return count;
@@ -1036,6 +1030,14 @@ void CStackBasicDescriptor::setType(const CCreature * c)
 	type = c;
 }
 
+bool operator== (const CStackBasicDescriptor & l, const CStackBasicDescriptor & r)
+{
+	return (!l.type && !r.type)
+	|| (l.type && r.type
+		&& l.type->getId() == r.type->getId()
+		&& l.count == r.count);
+}
+
 void CStackBasicDescriptor::serializeJson(JsonSerializeFormat & handler)
 {
 	handler.serializeInt("amount", count);
@@ -1044,27 +1046,27 @@ void CStackBasicDescriptor::serializeJson(JsonSerializeFormat & handler)
 	{
 		if(type)
 		{
-			std::string typeName = type->identifier;
+			std::string typeName = type->getJsonKey();
 			handler.serializeString("type", typeName);
 		}
 	}
 	else
 	{
-		std::string typeName("");
+		std::string typeName;
 		handler.serializeString("type", typeName);
 		if(!typeName.empty())
-			setType(VLC->creh->getCreature(CModHandler::scopeMap(), typeName));
+			setType(VLC->creh->getCreature(ModScope::scopeMap(), typeName));
 	}
 }
 
-void CSimpleArmy::clear()
+void CSimpleArmy::clearSlots()
 {
 	army.clear();
 }
 
 CSimpleArmy::operator bool() const
 {
-	return army.size();
+	return !army.empty();
 }
 
 bool CSimpleArmy::setCreature(SlotID slot, CreatureID cre, TQuantity count)
