@@ -21,7 +21,7 @@ namespace LogicalExpressionDetail
 	class ExpressionBase
 	{
 	public:
-		/// Possible logical operations, mostly needed to create different types for boost::variant
+		/// Possible logical operations, mostly needed to create different types for std::variant
 		enum EOperations
 		{
 			ANY_OF,
@@ -30,19 +30,14 @@ namespace LogicalExpressionDetail
 		};
 		template<EOperations tag> class Element;
 
-		typedef Element<ANY_OF> OperatorAny;
-		typedef Element<ALL_OF> OperatorAll;
-		typedef Element<NONE_OF> OperatorNone;
+		using OperatorAny = Element<ANY_OF>;
+		using OperatorAll = Element<ALL_OF>;
+		using OperatorNone = Element<NONE_OF>;
 
-		typedef ContainedClass Value;
+		using Value = ContainedClass;
 
 		/// Variant that contains all possible elements from logical expression
-		typedef boost::variant<
-			OperatorAll,
-			OperatorAny,
-			OperatorNone,
-			Value
-			> Variant;
+		using Variant = std::variant<OperatorAll, OperatorAny, OperatorNone, Value>;
 
 		/// Variant element, contains list of expressions to which operation "tag" should be applied
 		template<EOperations tag>
@@ -70,10 +65,10 @@ namespace LogicalExpressionDetail
 	};
 
 	/// Visitor to test result (true/false) of the expression
-	template <typename ContainedClass>
-	class TestVisitor : public boost::static_visitor<bool>
+	template<typename ContainedClass>
+	class TestVisitor
 	{
-		typedef ExpressionBase<ContainedClass> Base;
+		using Base = ExpressionBase<ContainedClass>;
 
 		std::function<bool(const typename Base::Value &)> classTest;
 
@@ -81,7 +76,7 @@ namespace LogicalExpressionDetail
 		{
 			return boost::range::count_if(element, [&](const typename Base::Variant & expr)
 			{
-				return boost::apply_visitor(*this, expr);
+				return std::visit(*this, expr);
 			});
 		}
 	public:
@@ -116,10 +111,10 @@ namespace LogicalExpressionDetail
 	template <typename ContainedClass>
 	class FalsifiabilityVisitor;
 
-	template <typename ContainedClass>
-	class PossibilityVisitor : public boost::static_visitor<bool>
+	template<typename ContainedClass>
+	class PossibilityVisitor
 	{
-		typedef ExpressionBase<ContainedClass> Base;
+		using Base = ExpressionBase<ContainedClass>;
 
 	protected:
 		std::function<bool(const typename Base::Value &)> satisfiabilityTest;
@@ -131,7 +126,7 @@ namespace LogicalExpressionDetail
 		{
 			return boost::range::count_if(element, [&](const typename Base::Variant & expr)
 			{
-				return boost::apply_visitor(*satisfiabilityVisitor, expr);
+				return std::visit(*satisfiabilityVisitor, expr);
 			});
 		}
 
@@ -139,7 +134,7 @@ namespace LogicalExpressionDetail
 		{
 			return boost::range::count_if(element, [&](const typename Base::Variant & expr)
 			{
-				return boost::apply_visitor(*falsifiabilityVisitor, expr);
+				return std::visit(*falsifiabilityVisitor, expr);
 			});
 		}
 
@@ -167,7 +162,7 @@ namespace LogicalExpressionDetail
 	template <typename ContainedClass>
 	class SatisfiabilityVisitor : public PossibilityVisitor<ContainedClass>
 	{
-		typedef ExpressionBase<ContainedClass> Base;
+		using Base = ExpressionBase<ContainedClass>;
 
 	public:
 		SatisfiabilityVisitor(std::function<bool (const typename Base::Value &)> satisfiabilityTest,
@@ -202,7 +197,7 @@ namespace LogicalExpressionDetail
 	template <typename ContainedClass>
 	class FalsifiabilityVisitor : public PossibilityVisitor<ContainedClass>
 	{
-		typedef ExpressionBase<ContainedClass> Base;
+		using Base = ExpressionBase<ContainedClass>;
 
 	public:
 		FalsifiabilityVisitor(std::function<bool (const typename Base::Value &)> satisfiabilityTest,
@@ -235,11 +230,11 @@ namespace LogicalExpressionDetail
 
 	/// visitor that is trying to generates candidates that must be fulfilled
 	/// to complete this expression
-	template <typename ContainedClass>
-	class CandidatesVisitor : public boost::static_visitor<std::vector<ContainedClass> >
+	template<typename ContainedClass>
+	class CandidatesVisitor
 	{
-		typedef ExpressionBase<ContainedClass> Base;
-		typedef std::vector<typename Base::Value> TValueList;
+		using Base = ExpressionBase<ContainedClass>;
+		using TValueList = std::vector<typename Base::Value>;
 
 		TestVisitor<ContainedClass> classTest;
 
@@ -254,7 +249,7 @@ namespace LogicalExpressionDetail
 			if (!classTest(element))
 			{
 				for (auto & elem : element.expressions)
-					boost::range::copy(boost::apply_visitor(*this, elem), std::back_inserter(ret));
+					boost::range::copy(std::visit(*this, elem), std::back_inserter(ret));
 			}
 			return ret;
 		}
@@ -265,7 +260,7 @@ namespace LogicalExpressionDetail
 			if (!classTest(element))
 			{
 				for (auto & elem : element.expressions)
-					boost::range::copy(boost::apply_visitor(*this, elem), std::back_inserter(ret));
+					boost::range::copy(std::visit(*this, elem), std::back_inserter(ret));
 			}
 			return ret;
 		}
@@ -285,10 +280,10 @@ namespace LogicalExpressionDetail
 	};
 
 	/// Simple foreach visitor
-	template <typename ContainedClass>
-	class ForEachVisitor : public boost::static_visitor<typename ExpressionBase<ContainedClass>::Variant>
+	template<typename ContainedClass>
+	class ForEachVisitor
 	{
-		typedef ExpressionBase<ContainedClass> Base;
+		using Base = ExpressionBase<ContainedClass>;
 
 		std::function<typename Base::Variant(const typename Base::Value &)> visitor;
 
@@ -306,16 +301,16 @@ namespace LogicalExpressionDetail
 		typename Base::Variant operator()(Type element) const
 		{
 			for (auto & entry : element.expressions)
-				entry = boost::apply_visitor(*this, entry);
+				entry = std::visit(*this, entry);
 			return element;
 		}
 	};
 
 	/// Minimizing visitor that removes all redundant elements from variant (e.g. AllOf inside another AllOf can be merged safely)
-	template <typename ContainedClass>
-	class MinimizingVisitor : public boost::static_visitor<typename ExpressionBase<ContainedClass>::Variant>
+	template<typename ContainedClass>
+	class MinimizingVisitor
 	{
-		typedef ExpressionBase<ContainedClass> Base;
+		using Base = ExpressionBase<ContainedClass>;
 
 	public:
 		typename Base::Variant operator()(const typename Base::Value & element) const
@@ -330,15 +325,15 @@ namespace LogicalExpressionDetail
 
 			for (auto & entryRO : element.expressions)
 			{
-				auto entry = boost::apply_visitor(*this, entryRO);
+				auto entry = std::visit(*this, entryRO);
 
 				try
 				{
 					// copy entries from child of this type
-					auto sublist = boost::get<Type>(entry).expressions;
+					auto sublist = std::get<Type>(entry).expressions;
 					std::move(sublist.begin(), sublist.end(), std::back_inserter(ret.expressions));
 				}
-				catch (boost::bad_get &)
+				catch (std::bad_variant_access &)
 				{
 					// different type (e.g. allOf vs oneOf) just copy
 					ret.expressions.push_back(entry);
@@ -360,7 +355,7 @@ namespace LogicalExpressionDetail
 	template <typename ContainedClass>
 	class Reader
 	{
-		typedef ExpressionBase<ContainedClass> Base;
+		using Base = ExpressionBase<ContainedClass>;
 
 		std::function<typename Base::Value(const JsonNode &)> classParser;
 
@@ -397,10 +392,10 @@ namespace LogicalExpressionDetail
 	};
 
 	/// Serializes expression in JSON format. Part of map format.
-	template <typename ContainedClass>
-	class Writer : public boost::static_visitor<JsonNode>
+	template<typename ContainedClass>
+	class Writer
 	{
-		typedef ExpressionBase<ContainedClass> Base;
+		using Base = ExpressionBase<ContainedClass>;
 
 		std::function<JsonNode(const typename Base::Value &)> classPrinter;
 
@@ -410,7 +405,7 @@ namespace LogicalExpressionDetail
 			ret.Vector().resize(1);
 			ret.Vector().back().String() = name;
 			for (auto & expr : element)
-				ret.Vector().push_back(boost::apply_visitor(*this, expr));
+				ret.Vector().push_back(std::visit(*this, expr));
 			return ret;
 		}
 	public:
@@ -439,13 +434,13 @@ namespace LogicalExpressionDetail
 		}
 	};
 
-	std::string DLL_LINKAGE getTextForOperator(std::string operation);
+	std::string DLL_LINKAGE getTextForOperator(const std::string & operation);
 
 	/// Prints expression in human-readable format
-	template <typename ContainedClass>
-	class Printer : public boost::static_visitor<std::string>
+	template<typename ContainedClass>
+	class Printer
 	{
-		typedef ExpressionBase<ContainedClass> Base;
+		using Base = ExpressionBase<ContainedClass>;
 
 		std::function<std::string(const typename Base::Value &)> classPrinter;
 		std::unique_ptr<TestVisitor<ContainedClass>> statusTest;
@@ -465,7 +460,7 @@ namespace LogicalExpressionDetail
 			std::string ret;
 			prefix.push_back('\t');
 			for (auto & expr : element)
-				ret += prefix + boost::apply_visitor(*this, expr) + "\n";
+				ret += prefix + std::visit(*this, expr) + "\n";
 			prefix.pop_back();
 			return ret;
 		}
@@ -510,30 +505,27 @@ namespace LogicalExpressionDetail
 template<typename ContainedClass>
 class LogicalExpression
 {
-	typedef LogicalExpressionDetail::ExpressionBase<ContainedClass> Base;
+	using Base = LogicalExpressionDetail::ExpressionBase<ContainedClass>;
+
 public:
 	/// Type of values used in expressions, same as ContainedClass
-	typedef typename Base::Value Value;
+	using Value = typename Base::Value;
 	/// Operators for use in expressions, all include vectors with operands
-	typedef typename Base::OperatorAny OperatorAny;
-	typedef typename Base::OperatorAll OperatorAll;
-	typedef typename Base::OperatorNone OperatorNone;
+	using OperatorAny = typename Base::OperatorAny;
+	using OperatorAll = typename Base::OperatorAll;
+	using OperatorNone = typename Base::OperatorNone;
 	/// one expression entry
-	typedef typename Base::Variant Variant;
+	using Variant = typename Base::Variant;
 
 private:
 	Variant data;
 
 public:
 	/// Base constructor
-	LogicalExpression()
-	{}
+	LogicalExpression() = default;
 
 	/// Constructor from variant or (implicitly) from Operator* types
-	LogicalExpression(const Variant & data):
-		data(data)
-	{
-	}
+	LogicalExpression(const Variant & data): data(data) {}
 
 	/// Constructor that receives JsonNode as input and function that can parse Value instances
 	LogicalExpression(const JsonNode & input, std::function<Value(const JsonNode &)> parser)
@@ -552,14 +544,14 @@ public:
 	Variant morph(std::function<Variant(const Value &)> morpher) const
 	{
 		LogicalExpressionDetail::ForEachVisitor<Value> visitor(morpher);
-		return boost::apply_visitor(visitor, data);
+		return std::visit(visitor, data);
 	}
 
 	/// Minimizes expression, removing any redundant elements
 	void minimize()
 	{
 		LogicalExpressionDetail::MinimizingVisitor<Value> visitor;
-		data = boost::apply_visitor(visitor, data);
+		data = std::visit(visitor, data);
 	}
 
 	/// calculates if expression evaluates to "true".
@@ -567,7 +559,7 @@ public:
 	bool test(std::function<bool(const Value &)> toBool) const
 	{
 		LogicalExpressionDetail::TestVisitor<Value> testVisitor(toBool);
-		return boost::apply_visitor(testVisitor, data);
+		return std::visit(testVisitor, data);
 	}
 
 	/// calculates if expression can evaluate to "true".
@@ -579,7 +571,7 @@ public:
 		satisfiabilityVisitor.setFalsifiabilityVisitor(&falsifiabilityVisitor);
 		falsifiabilityVisitor.setSatisfiabilityVisitor(&satisfiabilityVisitor);
 
-		return boost::apply_visitor(satisfiabilityVisitor, data);
+		return std::visit(satisfiabilityVisitor, data);
 	}
 
 	/// calculates if expression can evaluate to "false".
@@ -591,14 +583,14 @@ public:
 		satisfiabilityVisitor.setFalsifiabilityVisitor(&falsifiabilityVisitor);
 		falsifiabilityVisitor.setFalsifiabilityVisitor(&satisfiabilityVisitor);
 
-		return boost::apply_visitor(falsifiabilityVisitor, data);
+		return std::visit(falsifiabilityVisitor, data);
 	}
 
 	/// generates list of candidates that can be fulfilled by caller (like AI)
 	std::vector<Value> getFulfillmentCandidates(std::function<bool(const Value &)> toBool) const
 	{
 		LogicalExpressionDetail::CandidatesVisitor<Value> candidateVisitor(toBool);
-		return boost::apply_visitor(candidateVisitor, data);
+		return std::visit(candidateVisitor, data);
 	}
 
 	/// Converts expression in human-readable form
@@ -607,18 +599,18 @@ public:
 	std::string toString(std::function<std::string(const Value &)> toStr) const
 	{
 		LogicalExpressionDetail::Printer<Value> printVisitor(toStr);
-		return boost::apply_visitor(printVisitor, data);
+		return std::visit(printVisitor, data);
 	}
 	std::string toString(std::function<std::string(const Value &)> toStr, std::function<bool(const Value &)> toBool) const
 	{
 		LogicalExpressionDetail::Printer<Value> printVisitor(toStr, toBool);
-		return boost::apply_visitor(printVisitor, data);
+		return std::visit(printVisitor, data);
 	}
 
 	JsonNode toJson(std::function<JsonNode(const Value &)> toJson) const
 	{
 		LogicalExpressionDetail::Writer<Value> writeVisitor(toJson);
-		return boost::apply_visitor(writeVisitor, data);
+		return std::visit(writeVisitor, data);
 	}
 
 	template <typename Handler>
