@@ -25,8 +25,6 @@ Destination::Destination()
 
 }
 
-Destination::~Destination() = default;
-
 Destination::Destination(const battle::Unit * destination)
 	: unitValue(destination),
 	hexValue(destination->getPosition())
@@ -47,22 +45,6 @@ Destination::Destination(const Unit * destination, const BattleHex & exactHex)
 {
 
 }
-
-Destination::Destination(const Destination & other)
-	: unitValue(other.unitValue),
-	hexValue(other.hexValue)
-{
-
-}
-
-Destination & Destination::operator=(const Destination & other)
-{
-	unitValue = other.unitValue;
-	hexValue = other.hexValue;
-	return *this;
-}
-
-
 }
 
 VCMI_LIB_NAMESPACE_END
