@@ -15,7 +15,7 @@ VCMI_LIB_NAMESPACE_BEGIN
 CFileInputStream::CFileInputStream(const boost::filesystem::path & file, si64 start, si64 size)
   : dataStart{start},
 	dataSize{size},
-	fileStream{file, std::ios::in | std::ios::binary}
+	fileStream{file.c_str(), std::ios::in | std::ios::binary}
 {
 	if (fileStream.fail())
 		throw std::runtime_error("File " + file.string() + " isn't available.");
