@@ -23,13 +23,12 @@ public:
 	AbilityCaster(const battle::Unit * actualCaster_, int32_t baseSpellLevel_);
 	virtual ~AbilityCaster();
 
-	int32_t getSpellSchoolLevel(const Spell * spell, int32_t * outSelectedSchool = nullptr) const override;
+	int32_t getSpellSchoolLevel(const Spell * spell, SpellSchool * outSelectedSchool = nullptr) const override;
 	int32_t getEffectLevel(const Spell * spell) const override;
 	void getCastDescription(const Spell * spell, const std::vector<const battle::Unit *> & attacked, MetaString & text) const override;
 	void spendMana(ServerCallback * server, const int32_t spellCost) const override;
 
 private:
-	const battle::Unit * actualCaster;
 	int32_t baseSpellLevel;
 };
 
