@@ -13,21 +13,19 @@
 
 #include <vcmi/spells/Spell.h>
 
-#include "../NetPacksBase.h"
-#include "../HeroBonus.h"
+#include "../MetaString.h"
 #include "../battle/Unit.h"
+#include "../bonuses/Bonus.h"
 
 VCMI_LIB_NAMESPACE_BEGIN
 
 namespace spells
 {
 
-BonusCaster::BonusCaster(const Caster * actualCaster_, std::shared_ptr<Bonus> bonus_)
-	: ProxyCaster(actualCaster_),
-	actualCaster(actualCaster_),
-	bonus(bonus_)
+BonusCaster::BonusCaster(const Caster * actualCaster_, std::shared_ptr<Bonus> bonus_):
+	ProxyCaster(actualCaster_),
+	bonus(std::move(bonus_))
 {
-
 }
 
 BonusCaster::~BonusCaster() = default;
@@ -35,7 +33,7 @@ BonusCaster::~BonusCaster() = default;
 void BonusCaster::getCasterName(MetaString & text) const
 {
 	if(!bonus->description.empty())
-		text.addReplacement(bonus->description);
+		text.replaceRawString(bonus->description);
 	else
 		actualCaster->getCasterName(text);
 }
@@ -45,9 +43,9 @@ void BonusCaster::getCastDescription(const Spell * spell, const std::vector<cons
 	const bool singleTarget = attacked.size() == 1;
 	const int textIndex = singleTarget ? 195 : 196;
 
-	text.addTxt(MetaString::GENERAL_TXT, textIndex);
+	text.appendLocalString(EMetaText::GENERAL_TXT, textIndex);
 	getCasterName(text);
-	text.addReplacement(MetaString::SPELL_NAME, spell->getIndex());
+	text.replaceName(spell->getId());
 	if(singleTarget)
 		attacked.at(0)->addNameReplacement(text, true);
 }
