@@ -12,7 +12,7 @@
 
 #include <vcmi/spells/Magic.h>
 
-#include "../NetPacksBase.h"
+#include "../MetaString.h"
 
 VCMI_LIB_NAMESPACE_BEGIN
 
@@ -24,9 +24,6 @@ namespace detail
 class DLL_LINKAGE ProblemImpl : public Problem
 {
 public:
-	ProblemImpl();
-	virtual ~ProblemImpl();
-
 	void add(MetaString && description, Severity severity = CRITICAL) override;
 
 	void getAll(std::vector<std::string> & target) const override;
