@@ -9,18 +9,35 @@
  */
 #include "StdInc.h"
 #include "inspector.h"
+#include "../lib/ArtifactUtils.h"
 #include "../lib/CArtHandler.h"
 #include "../lib/spells/CSpellHandler.h"
 #include "../lib/CHeroHandler.h"
 #include "../lib/CRandomGenerator.h"
-#include "../lib/mapObjects/CObjectClassesHandler.h"
+#include "../lib/mapObjectConstructors/AObjectTypeHandler.h"
+#include "../lib/mapObjectConstructors/CObjectClassesHandler.h"
+#include "../lib/mapObjects/ObjectTemplate.h"
 #include "../lib/mapping/CMap.h"
+#include "../lib/constants/StringConstants.h"
 
 #include "townbulidingswidget.h"
 #include "armywidget.h"
 #include "messagewidget.h"
 #include "rewardswidget.h"
 #include "questwidget.h"
+#include "heroskillswidget.h"
+#include "portraitwidget.h"
+#include "PickObjectDelegate.h"
+#include "../mapcontroller.h"
+
+static QList<std::pair<QString, QVariant>> CharacterIdentifiers
+{
+	{QObject::tr("Compliant"), QVariant::fromValue(int(CGCreature::Character::COMPLIANT))},
+	{QObject::tr("Friendly"), QVariant::fromValue(int(CGCreature::Character::FRIENDLY))},
+	{QObject::tr("Aggressive"), QVariant::fromValue(int(CGCreature::Character::AGGRESSIVE))},
+	{QObject::tr("Hostile"), QVariant::fromValue(int(CGCreature::Character::HOSTILE))},
+	{QObject::tr("Savage"), QVariant::fromValue(int(CGCreature::Character::SAVAGE))},
+};
 
 //===============IMPLEMENT OBJECT INITIALIZATION FUNCTIONS================
 Initializer::Initializer(CGObjectInstance * o, const PlayerColor & pl) : defaultPlayer(pl)
@@ -36,22 +53,16 @@ Initializer::Initializer(CGObjectInstance * o, const PlayerColor & pl) : default
 	INIT_OBJ_TYPE(CGDwelling);
 	INIT_OBJ_TYPE(CGTownInstance);
 	INIT_OBJ_TYPE(CGCreature);
+	INIT_OBJ_TYPE(CGHeroPlaceholder);
 	INIT_OBJ_TYPE(CGHeroInstance);
 	INIT_OBJ_TYPE(CGSignBottle);
 	INIT_OBJ_TYPE(CGLighthouse);
+	//INIT_OBJ_TYPE(CRewardableObject);
 	//INIT_OBJ_TYPE(CGPandoraBox);
 	//INIT_OBJ_TYPE(CGEvent);
 	//INIT_OBJ_TYPE(CGSeerHut);
 }
 
-bool stringToBool(const QString & s)
-{
-	if(s == "TRUE")
-		return true;
-	//if(s == "FALSE")
-	return false;
-}
-
 void Initializer::initialize(CArmedInstance * o)
 {
 	if(!o) return;
@@ -67,7 +78,8 @@ void Initializer::initialize(CGCreature * o)
 	if(!o) return;
 	
 	o->character = CGCreature::Character::HOSTILE;
-	o->putStack(SlotID(0), new CStackInstance(CreatureID(o->subID), 0, false));
+	if(!o->hasStackAtSlot(SlotID(0)))
+	   o->putStack(SlotID(0), new CStackInstance(CreatureID(o->subID), 0, false));
 }
 
 void Initializer::initialize(CGDwelling * o)
@@ -75,14 +87,6 @@ void Initializer::initialize(CGDwelling * o)
 	if(!o) return;
 	
 	o->tempOwner = defaultPlayer;
-	
-	switch(o->ID)
-	{
-		case Obj::RANDOM_DWELLING:
-		case Obj::RANDOM_DWELLING_LVL:
-		case Obj::RANDOM_DWELLING_FACTION:
-			o->initRandomObjectInfo();
-	}
 }
 
 void Initializer::initialize(CGGarrison * o)
@@ -107,34 +111,50 @@ void Initializer::initialize(CGLighthouse * o)
 	o->tempOwner = defaultPlayer;
 }
 
-void Initializer::initialize(CGHeroInstance * o)
+void Initializer::initialize(CGHeroPlaceholder * o)
 {
 	if(!o) return;
 	
+	o->tempOwner = defaultPlayer;
+	
+	if(!o->powerRank.has_value() && !o->heroType.has_value())
+		o->powerRank = 0;
+	
+	if(o->powerRank.has_value() && o->heroType.has_value())
+		o->powerRank.reset();
+}
+
+void Initializer::initialize(CGHeroInstance * o)
+{
+	if(!o)
+		return;
+	
 	o->tempOwner = defaultPlayer;
 	if(o->ID == Obj::PRISON)
+	{
+		o->subID = 0;
 		o->tempOwner = PlayerColor::NEUTRAL;
+	}
 	
 	if(o->ID == Obj::HERO)
 	{
 		for(auto t : VLC->heroh->objects)
 		{
-			if(t->heroClass == VLC->heroh->classes.objects[o->subID].get())
+			if(t->heroClass->getId() == HeroClassID(o->subID))
 			{
-				o->type = VLC->heroh->objects[o->subID];
+				o->type = t;
 				break;
 			}
 		}
 	}
 	
-	if(!o->type)
-		o->type = VLC->heroh->objects.at(o->subID);
-	
-	o->name = o->type->getName();
-	o->sex = o->type->sex;
-	o->biography = o->type->biography;
-	o->portrait = o->type->imageIndex;
-	o->randomizeArmy(o->type->heroClass->faction);
+	if(o->type)
+	{
+		//	o->type = VLC->heroh->objects.at(o->subID);
+		
+		o->gender = o->type->gender;
+		o->randomizeArmy(o->type->heroClass->faction);
+	}
 }
 
 void Initializer::initialize(CGTownInstance * o)
@@ -166,12 +186,12 @@ void Initializer::initialize(CGArtifact * o)
 		std::vector<SpellID> out;
 		for(auto spell : VLC->spellh->objects) //spellh size appears to be greater (?)
 		{
-			//if(map->isAllowedSpell(spell->id))
+			if(VLC->spellh->getDefaultAllowed().count(spell->id) != 0)
 			{
 				out.push_back(spell->id);
 			}
 		}
-		auto a = CArtifactInstance::createScroll(*RandomGeneratorUtil::nextItem(out, CRandomGenerator::getDefault()));
+		auto a = ArtifactUtils::createScroll(*RandomGeneratorUtil::nextItem(out, CRandomGenerator::getDefault()));
 		o->storedArtifact = a;
 	}
 }
@@ -181,8 +201,16 @@ void Initializer::initialize(CGMine * o)
 	if(!o) return;
 	
 	o->tempOwner = defaultPlayer;
-	o->producedResource = Res::ERes(o->subID);
-	o->producedQuantity = o->defaultResProduction();
+	if(o->isAbandoned())
+	{
+		for(auto r = GameResID(0); r < GameResID::COUNT; ++r)
+			o->abandonedMineResources.insert(r);
+	}
+	else
+	{
+		o->producedResource = GameResID(o->subID);
+		o->producedQuantity = o->defaultResProduction();
+	}
 }
 
 void Initializer::initialize(CGResource * o)
@@ -206,6 +234,12 @@ void Inspector::updateProperties(CGDwelling * o)
 	if(!o) return;
 	
 	addProperty("Owner", o->tempOwner, false);
+	
+	if (o->ID == Obj::RANDOM_DWELLING || o->ID == Obj::RANDOM_DWELLING_LVL)
+	{
+		auto * delegate = new PickObjectDelegate(controller, PickObjectDelegate::typedFilter<CGTownInstance>);
+		addProperty("Same as town", PropertyEditorPlaceholder(), delegate, false);
+	}
 }
 
 void Inspector::updateProperties(CGLighthouse * o)
@@ -220,7 +254,7 @@ void Inspector::updateProperties(CGGarrison * o)
 	if(!o) return;
 	
 	addProperty("Owner", o->tempOwner, false);
-	addProperty("Removable units", o->removableUnits, InspectorDelegate::boolDelegate(), false);
+	addProperty("Removable units", o->removableUnits, false);
 }
 
 void Inspector::updateProperties(CGShipyard * o)
@@ -230,34 +264,70 @@ void Inspector::updateProperties(CGShipyard * o)
 	addProperty("Owner", o->tempOwner, false);
 }
 
+void Inspector::updateProperties(CGHeroPlaceholder * o)
+{
+	if(!o) return;
+	
+	addProperty("Owner", o->tempOwner, false);
+	
+	bool type = false;
+	if(o->heroType.has_value())
+		type = true;
+	else if(!o->powerRank.has_value())
+		assert(0); //one of values must be initialized
+	
+	{
+		auto * delegate = new InspectorDelegate;
+		delegate->options = {{"POWER RANK", QVariant::fromValue(false)}, {"HERO TYPE", QVariant::fromValue(true)}};
+		addProperty("Placeholder type", delegate->options[type].first, delegate, false);
+	}
+	
+	addProperty("Power rank", o->powerRank.has_value() ? o->powerRank.value() : 0, type);
+	
+	{
+		auto * delegate = new InspectorDelegate;
+		for(int i = 0; i < VLC->heroh->objects.size(); ++i)
+		{
+			delegate->options.push_back({QObject::tr(VLC->heroh->objects[i]->getNameTranslated().c_str()), QVariant::fromValue(VLC->heroh->objects[i]->getId().getNum())});
+		}
+		addProperty("Hero type", o->heroType.has_value() ? VLC->heroh->getById(o->heroType.value())->getNameTranslated() : "", delegate, !type);
+	}
+}
+
 void Inspector::updateProperties(CGHeroInstance * o)
 {
 	if(!o) return;
 	
 	addProperty("Owner", o->tempOwner, o->ID == Obj::PRISON); //field is not editable for prison
 	addProperty<int>("Experience", o->exp, false);
-	addProperty("Hero class", o->type->heroClass->getName(), true);
+	addProperty("Hero class", o->type ? o->type->heroClass->getNameTranslated() : "", true);
 	
-	{ //Sex
+	{ //Gender
 		auto * delegate = new InspectorDelegate;
-		delegate->options << "MALE" << "FEMALE";
-		addProperty<std::string>("Sex", (o->sex ? "FEMALE" : "MALE"), delegate , false);
+		delegate->options = {{"MALE", QVariant::fromValue(int(EHeroGender::MALE))}, {"FEMALE", QVariant::fromValue(int(EHeroGender::FEMALE))}};
+		addProperty<std::string>("Gender", (o->gender == EHeroGender::FEMALE ? "FEMALE" : "MALE"), delegate , false);
 	}
-	addProperty("Name", o->name, false);
-	addProperty("Biography", o->biography, new MessageDelegate, false);
-	addProperty("Portrait", o->portrait, false);
+	addProperty("Name", o->getNameTranslated(), false);
+	addProperty("Biography", o->getBiographyTranslated(), new MessageDelegate, false);
+	addProperty("Portrait", PropertyEditorPlaceholder(), new PortraitDelegate(*o), false);
 	
+	auto * delegate = new HeroSkillsDelegate(*o);
+	addProperty("Skills", PropertyEditorPlaceholder(), delegate, false);
+	
+	if(o->type)
 	{ //Hero type
 		auto * delegate = new InspectorDelegate;
 		for(int i = 0; i < VLC->heroh->objects.size(); ++i)
 		{
-			if(map->allowedHeroes.at(i))
+			if(controller.map()->allowedHeroes.count(HeroTypeID(i)) != 0)
 			{
 				if(o->ID == Obj::PRISON || (o->type && VLC->heroh->objects[i]->heroClass->getIndex() == o->type->heroClass->getIndex()))
-					delegate->options << QObject::tr(VLC->heroh->objects[i]->getName().c_str());
+				{
+					delegate->options.push_back({QObject::tr(VLC->heroh->objects[i]->getNameTranslated().c_str()), QVariant::fromValue(VLC->heroh->objects[i]->getId().getNum())});
+				}
 			}
 		}
-		addProperty("Hero type", o->type->getName(), delegate, false);
+		addProperty("Hero type", o->type->getNameTranslated(), delegate, false);
 	}
 }
 
@@ -265,7 +335,7 @@ void Inspector::updateProperties(CGTownInstance * o)
 {
 	if(!o) return;
 	
-	addProperty("Town name", o->name, false);
+	addProperty("Town name", o->getNameTranslated(), false);
 	
 	auto * delegate = new TownBuildingsDelegate(*o);
 	addProperty("Buildings", PropertyEditorPlaceholder(), delegate, false);
@@ -280,16 +350,16 @@ void Inspector::updateProperties(CGArtifact * o)
 	CArtifactInstance * instance = o->storedArtifact;
 	if(instance)
 	{
-		SpellID spellId = instance->getGivenSpellID();
-		if(spellId != -1)
+		SpellID spellId = instance->getScrollSpellID();
+		if(spellId != SpellID::NONE)
 		{
 			auto * delegate = new InspectorDelegate;
 			for(auto spell : VLC->spellh->objects)
 			{
-				//if(map->isAllowedSpell(spell->id))
-				delegate->options << QObject::tr(spell->name.c_str());
+				if(controller.map()->allowedSpells.count(spell->id) != 0)
+					delegate->options.push_back({QObject::tr(spell->getNameTranslated().c_str()), QVariant::fromValue(int(spell->getId()))});
 			}
-			addProperty("Spell", VLC->spellh->objects[spellId]->name, delegate, false);
+			addProperty("Spell", VLC->spellh->getById(spellId)->getNameTranslated(), delegate, false);
 		}
 	}
 }
@@ -325,60 +395,64 @@ void Inspector::updateProperties(CGCreature * o)
 	addProperty("Message", o->message, false);
 	{ //Character
 		auto * delegate = new InspectorDelegate;
-		delegate->options << "COMPLIANT" << "FRIENDLY" << "AGRESSIVE" << "HOSTILE" << "SAVAGE";
+		delegate->options = CharacterIdentifiers;
 		addProperty<CGCreature::Character>("Character", (CGCreature::Character)o->character, delegate, false);
 	}
-	addProperty("Never flees", o->neverFlees, InspectorDelegate::boolDelegate(), false);
-	addProperty("Not growing", o->notGrowingTeam, InspectorDelegate::boolDelegate(), false);
+	addProperty("Never flees", o->neverFlees, false);
+	addProperty("Not growing", o->notGrowingTeam, false);
 	addProperty("Artifact reward", o->gainedArtifact); //TODO: implement in setProperty
 	addProperty("Army", PropertyEditorPlaceholder(), true);
 	addProperty("Amount", o->stacks[SlotID(0)]->count, false);
 	//addProperty("Resources reward", o->resources); //TODO: implement in setProperty
 }
 
+void Inspector::updateProperties(CRewardableObject * o)
+{
+	if(!o) return;
+		
+	auto * delegate = new RewardsDelegate(*controller.map(), *o);
+	addProperty("Reward", PropertyEditorPlaceholder(), delegate, false);
+}
+
 void Inspector::updateProperties(CGPandoraBox * o)
 {
 	if(!o) return;
 	
 	addProperty("Message", o->message, new MessageDelegate, false);
-	
-	auto * delegate = new RewardsPandoraDelegate(*map, *o);
-	addProperty("Reward", PropertyEditorPlaceholder(), delegate, false);
 }
 
 void Inspector::updateProperties(CGEvent * o)
 {
 	if(!o) return;
 	
-	addProperty("Remove after", o->removeAfterVisit, InspectorDelegate::boolDelegate(), false);
-	addProperty("Human trigger", o->humanActivate, InspectorDelegate::boolDelegate(), false);
-	addProperty("Cpu trigger", o->computerActivate, InspectorDelegate::boolDelegate(), false);
+	addProperty("Remove after", o->removeAfterVisit, false);
+	addProperty("Human trigger", o->humanActivate, false);
+	addProperty("Cpu trigger", o->computerActivate, false);
 	//ui8 availableFor; //players whom this event is available for
 }
 
 void Inspector::updateProperties(CGSeerHut * o)
 {
-	if(!o) return;
-
-	{ //Mission type
-		auto * delegate = new InspectorDelegate;
-		delegate->options << "Reach level" << "Stats" << "Kill hero" << "Kill creature" << "Artifact" << "Army" << "Resources" << "Hero" << "Player";
-		addProperty<CQuest::Emission>("Mission type", o->quest->missionType, delegate, false);
-	}
+	if(!o || !o->quest) return;
 	
 	addProperty("First visit text", o->quest->firstVisitText, new MessageDelegate, false);
 	addProperty("Next visit text", o->quest->nextVisitText, new MessageDelegate, false);
 	addProperty("Completed text", o->quest->completedText, new MessageDelegate, false);
+	addProperty("Repeat quest", o->quest->repeatedQuest, false);
+	addProperty("Time limit", o->quest->lastDay, false);
 	
 	{ //Quest
-		auto * delegate = new QuestDelegate(*map, *o);
+		auto * delegate = new QuestDelegate(controller, *o->quest);
 		addProperty("Quest", PropertyEditorPlaceholder(), delegate, false);
 	}
+}
+
+void Inspector::updateProperties(CGQuestGuard * o)
+{
+	if(!o || !o->quest) return;
 	
-	{ //Reward
-		auto * delegate = new RewardsSeerhutDelegate(*map, *o);
-		addProperty("Reward", PropertyEditorPlaceholder(), delegate, false);
-	}
+	addProperty("Reward", PropertyEditorPlaceholder(), nullptr, true);
+	addProperty("Repeat quest", o->quest->repeatedQuest, true);
 }
 
 void Inspector::updateProperties()
@@ -401,10 +475,10 @@ void Inspector::updateProperties()
 	}
 	
 	auto * delegate = new InspectorDelegate();
-	delegate->options << "NEUTRAL";
-	for(int p = 0; p < map->players.size(); ++p)
-		if(map->players[p].canAnyonePlay())
-			delegate->options << QString("PLAYER %1").arg(p);
+	delegate->options.push_back({QObject::tr("neutral"), QVariant::fromValue(PlayerColor::NEUTRAL.getNum())});
+	for(int p = 0; p < controller.map()->players.size(); ++p)
+		if(controller.map()->players[p].canAnyonePlay())
+			delegate->options.push_back({QString::fromStdString(GameConstants::PLAYER_COLOR_NAMES[p]), QVariant::fromValue(PlayerColor(p).getNum())});
 	addProperty("Owner", obj->tempOwner, delegate, true);
 	
 	UPDATE_OBJ_PROPERTIES(CArmedInstance);
@@ -416,31 +490,44 @@ void Inspector::updateProperties()
 	UPDATE_OBJ_PROPERTIES(CGDwelling);
 	UPDATE_OBJ_PROPERTIES(CGTownInstance);
 	UPDATE_OBJ_PROPERTIES(CGCreature);
+	UPDATE_OBJ_PROPERTIES(CGHeroPlaceholder);
 	UPDATE_OBJ_PROPERTIES(CGHeroInstance);
 	UPDATE_OBJ_PROPERTIES(CGSignBottle);
 	UPDATE_OBJ_PROPERTIES(CGLighthouse);
+	UPDATE_OBJ_PROPERTIES(CRewardableObject);
 	UPDATE_OBJ_PROPERTIES(CGPandoraBox);
 	UPDATE_OBJ_PROPERTIES(CGEvent);
 	UPDATE_OBJ_PROPERTIES(CGSeerHut);
+	UPDATE_OBJ_PROPERTIES(CGQuestGuard);
 	
 	table->show();
 }
 
 //===============IMPLEMENT PROPERTY UPDATE================================
+void Inspector::setProperty(const QString & key, const QTableWidgetItem * item)
+{
+	if(!item->data(Qt::UserRole).isNull())
+	{
+		setProperty(key, item->data(Qt::UserRole));
+		return;
+	}
+	
+	if(item->flags() & Qt::ItemIsUserCheckable)
+	{
+		setProperty(key, QVariant::fromValue(item->checkState() == Qt::Checked));
+		return;
+	}
+	
+	setProperty(key, item->text());
+}
+
 void Inspector::setProperty(const QString & key, const QVariant & value)
 {
 	if(!obj)
 		return;
 	
 	if(key == "Owner")
-	{
-		PlayerColor owner(value.toString().mid(6).toInt()); //receiving PLAYER N, N has index 6
-		if(value == "NEUTRAL")
-			owner = PlayerColor::NEUTRAL;
-		if(value == "UNFLAGGABLE")
-			owner = PlayerColor::UNFLAGGABLE;
-		obj->tempOwner = owner;
-	}
+		obj->tempOwner = PlayerColor(value.toInt());
 	
 	SET_PROPERTIES(CArmedInstance);
 	SET_PROPERTIES(CGTownInstance);
@@ -450,13 +537,16 @@ void Inspector::setProperty(const QString & key, const QVariant & value)
 	SET_PROPERTIES(CGDwelling);
 	SET_PROPERTIES(CGGarrison);
 	SET_PROPERTIES(CGCreature);
+	SET_PROPERTIES(CGHeroPlaceholder);
 	SET_PROPERTIES(CGHeroInstance);
 	SET_PROPERTIES(CGShipyard);
 	SET_PROPERTIES(CGSignBottle);
 	SET_PROPERTIES(CGLighthouse);
+	SET_PROPERTIES(CRewardableObject);
 	SET_PROPERTIES(CGPandoraBox);
 	SET_PROPERTIES(CGEvent);
 	SET_PROPERTIES(CGSeerHut);
+	SET_PROPERTIES(CGQuestGuard);
 }
 
 void Inspector::setProperty(CArmedInstance * o, const QString & key, const QVariant & value)
@@ -469,9 +559,18 @@ void Inspector::setProperty(CGLighthouse * o, const QString & key, const QVarian
 	if(!o) return;
 }
 
+void Inspector::setProperty(CRewardableObject * o, const QString & key, const QVariant & value)
+{
+	if(!o) return;
+}
+
 void Inspector::setProperty(CGPandoraBox * o, const QString & key, const QVariant & value)
 {
 	if(!o) return;
+	
+	if(key == "Message")
+		o->message = MetaString::createFromTextID(mapRegisterLocalizedString("map", *controller.map(),
+			TextIdentifier("guards", o->instanceName, "message"), value.toString().toStdString()));
 }
 
 void Inspector::setProperty(CGEvent * o, const QString & key, const QVariant & value)
@@ -479,13 +578,13 @@ void Inspector::setProperty(CGEvent * o, const QString & key, const QVariant & v
 	if(!o) return;
 	
 	if(key == "Remove after")
-		o->removeAfterVisit = stringToBool(value.toString());
+		o->removeAfterVisit = value.toBool();
 	
 	if(key == "Human trigger")
-		o->humanActivate = stringToBool(value.toString());
+		o->humanActivate = value.toBool();
 	
 	if(key == "Cpu trigger")
-		o->computerActivate = stringToBool(value.toString());
+		o->computerActivate = value.toBool();
 }
 
 void Inspector::setProperty(CGTownInstance * o, const QString & key, const QVariant & value)
@@ -493,7 +592,8 @@ void Inspector::setProperty(CGTownInstance * o, const QString & key, const QVari
 	if(!o) return;
 	
 	if(key == "Town name")
-		o->name = value.toString().toStdString();
+		o->setNameTextId(mapRegisterLocalizedString("map", *controller.map(),
+			TextIdentifier("town", o->instanceName, "name"), value.toString().toStdString()));
 }
 
 void Inspector::setProperty(CGSignBottle * o, const QString & key, const QVariant & value)
@@ -501,7 +601,8 @@ void Inspector::setProperty(CGSignBottle * o, const QString & key, const QVarian
 	if(!o) return;
 	
 	if(key == "Message")
-		o->message = value.toString().toStdString();
+		o->message = MetaString::createFromTextID(mapRegisterLocalizedString("map", *controller.map(),
+			TextIdentifier("sign", o->instanceName, "message"), value.toString().toStdString()));
 }
 
 void Inspector::setProperty(CGMine * o, const QString & key, const QVariant & value)
@@ -517,24 +618,28 @@ void Inspector::setProperty(CGArtifact * o, const QString & key, const QVariant
 	if(!o) return;
 	
 	if(key == "Message")
-		o->message = value.toString().toStdString();
+		o->message = MetaString::createFromTextID(mapRegisterLocalizedString("map", *controller.map(),
+			TextIdentifier("guards", o->instanceName, "message"), value.toString().toStdString()));
 	
 	if(o->storedArtifact && key == "Spell")
 	{
-		for(auto spell : VLC->spellh->objects)
-		{
-			if(spell->name == value.toString().toStdString())
-			{
-				o->storedArtifact = CArtifactInstance::createScroll(spell->getId());
-				break;
-			}
-		}
+		o->storedArtifact = ArtifactUtils::createScroll(SpellID(value.toInt()));
 	}
 }
 
 void Inspector::setProperty(CGDwelling * o, const QString & key, const QVariant & value)
 {
 	if(!o) return;
+	
+	if(key == "Same as town")
+	{
+		if (!o->randomizationInfo.has_value())
+			o->randomizationInfo = CGDwellingRandomizationInfo();
+
+		o->randomizationInfo->instanceId = "";
+		if(CGTownInstance * town = data_cast<CGTownInstance>(value.toLongLong()))
+			o->randomizationInfo->instanceId = town->instanceName;
+	}
 }
 
 void Inspector::setProperty(CGGarrison * o, const QString & key, const QVariant & value)
@@ -542,33 +647,69 @@ void Inspector::setProperty(CGGarrison * o, const QString & key, const QVariant
 	if(!o) return;
 	
 	if(key == "Removable units")
-		o->removableUnits = stringToBool(value.toString());
+		o->removableUnits = value.toBool();
+}
+
+void Inspector::setProperty(CGHeroPlaceholder * o, const QString & key, const QVariant & value)
+{
+	if(!o) return;
+
+	if(key == "Placeholder type")
+	{
+		if(value.toBool())
+		{
+			if(!o->heroType.has_value())
+				o->heroType = HeroTypeID(0);
+			o->powerRank.reset();
+		}
+		else
+		{
+			if(!o->powerRank.has_value())
+				o->powerRank = 0;
+			o->heroType.reset();
+		}
+		
+		updateProperties();
+	}
+	
+	if(key == "Power rank")
+		o->powerRank = value.toInt();
+	
+	if(key == "Hero type")
+	{
+		o->heroType = HeroTypeID(value.toInt());
+	}
 }
 
 void Inspector::setProperty(CGHeroInstance * o, const QString & key, const QVariant & value)
 {
 	if(!o) return;
 	
-	if(key == "Sex")
-		o->sex = value.toString() == "MALE" ? 0 : 1;
+	if(key == "Gender")
+		o->gender = EHeroGender(value.toInt());
 	
 	if(key == "Name")
-		o->name = value.toString().toStdString();
+		o->nameCustomTextId = mapRegisterLocalizedString("map", *controller.map(),
+			TextIdentifier("hero", o->instanceName, "name"), value.toString().toStdString());
+	
+	if(key == "Biography")
+		o->biographyCustomTextId = mapRegisterLocalizedString("map", *controller.map(),
+			TextIdentifier("hero", o->instanceName, "biography"), value.toString().toStdString());
 	
 	if(key == "Experience")
-		o->exp = value.toInt();
+		o->exp = value.toString().toInt();
 	
 	if(key == "Hero type")
 	{
 		for(auto t : VLC->heroh->objects)
 		{
-			if(t->getName() == value.toString().toStdString())
+			if(t->getId() == value.toInt())
+			{
+				o->subID = value.toInt();
 				o->type = t.get();
+			}
 		}
-		o->name = o->type->getName();
-		o->sex = o->type->sex;
-		o->biography = o->type->biography;
-		o->portrait = o->type->imageIndex;
+		o->gender = o->type->gender;
 		o->randomizeArmy(o->type->heroClass->faction);
 		updateProperties(); //updating other properties after change
 	}
@@ -592,25 +733,14 @@ void Inspector::setProperty(CGCreature * o, const QString & key, const QVariant
 	if(!o) return;
 	
 	if(key == "Message")
-		o->message = value.toString().toStdString();
+		o->message = MetaString::createFromTextID(mapRegisterLocalizedString("map", *controller.map(),
+			TextIdentifier("monster", o->instanceName, "message"), value.toString().toStdString()));
 	if(key == "Character")
-	{
-		//COMPLIANT = 0, FRIENDLY = 1, AGRESSIVE = 2, HOSTILE = 3, SAVAGE = 4
-		if(value == "COMPLIANT")
-			o->character = CGCreature::Character::COMPLIANT;
-		if(value == "FRIENDLY")
-			o->character = CGCreature::Character::FRIENDLY;
-		if(value == "AGRESSIVE")
-			o->character = CGCreature::Character::AGRESSIVE;
-		if(value == "HOSTILE")
-			o->character = CGCreature::Character::HOSTILE;
-		if(value == "SAVAGE")
-			o->character = CGCreature::Character::SAVAGE;
-	}
+		o->character = CGCreature::Character(value.toInt());
 	if(key == "Never flees")
-		o->neverFlees = stringToBool(value.toString());
+		o->neverFlees = value.toBool();
 	if(key == "Not growing")
-		o->notGrowingTeam = stringToBool(value.toString());
+		o->notGrowingTeam = value.toBool();
 	if(key == "Amount")
 		o->stacks[SlotID(0)]->count = value.toString().toInt();
 }
@@ -619,63 +749,64 @@ void Inspector::setProperty(CGSeerHut * o, const QString & key, const QVariant &
 {
 	if(!o) return;
 	
-	if(key == "Mission type")
-	{
-		if(value == "Reach level")
-			o->quest->missionType = CQuest::Emission::MISSION_LEVEL;
-		if(value == "Stats")
-			o->quest->missionType = CQuest::Emission::MISSION_PRIMARY_STAT;
-		if(value == "Kill hero")
-			o->quest->missionType = CQuest::Emission::MISSION_KILL_HERO;
-		if(value == "Kill creature")
-			o->quest->missionType = CQuest::Emission::MISSION_KILL_CREATURE;
-		if(value == "Artifact")
-			o->quest->missionType = CQuest::Emission::MISSION_ART;
-		if(value == "Army")
-			o->quest->missionType = CQuest::Emission::MISSION_ARMY;
-		if(value == "Resources")
-			o->quest->missionType = CQuest::Emission::MISSION_RESOURCES;
-		if(value == "Hero")
-			o->quest->missionType = CQuest::Emission::MISSION_HERO;
-		if(value == "Player")
-			o->quest->missionType = CQuest::Emission::MISSION_PLAYER;
-	}
-	
 	if(key == "First visit text")
-		o->quest->firstVisitText = value.toString().toStdString();
+		o->quest->firstVisitText = MetaString::createFromTextID(mapRegisterLocalizedString("map", *controller.map(),
+			TextIdentifier("quest", o->instanceName, "firstVisit"), value.toString().toStdString()));
 	if(key == "Next visit text")
-		o->quest->nextVisitText = value.toString().toStdString();
+		o->quest->nextVisitText = MetaString::createFromTextID(mapRegisterLocalizedString("map", *controller.map(),
+			TextIdentifier("quest", o->instanceName, "nextVisit"), value.toString().toStdString()));
 	if(key == "Completed text")
-		o->quest->completedText = value.toString().toStdString();
+		o->quest->completedText = MetaString::createFromTextID(mapRegisterLocalizedString("map", *controller.map(),
+			TextIdentifier("quest", o->instanceName, "completed"), value.toString().toStdString()));
+	if(key == "Repeat quest")
+		o->quest->repeatedQuest = value.toBool();
+	if(key == "Time limit")
+		o->quest->lastDay = value.toString().toInt();
+}
+
+void Inspector::setProperty(CGQuestGuard * o, const QString & key, const QVariant & value)
+{
+	if(!o) return;
 }
 
 
 //===============IMPLEMENT PROPERTY VALUE TYPE============================
 QTableWidgetItem * Inspector::addProperty(CGObjectInstance * value)
 {
-	return new QTableWidgetItem(QString::number(data_cast<CGObjectInstance>(value)));
+	auto * item = new QTableWidgetItem(QString::number(data_cast<CGObjectInstance>(value)));
+	item->setFlags(Qt::NoItemFlags);
+	return item;
 }
 
 QTableWidgetItem * Inspector::addProperty(Inspector::PropertyEditorPlaceholder value)
 {
-	auto item = new QTableWidgetItem("");
-	item->setData(Qt::UserRole, QString("PropertyEditor"));
+	auto item = new QTableWidgetItem("...");
+	item->setFlags(Qt::NoItemFlags);
 	return item;
 }
 
 QTableWidgetItem * Inspector::addProperty(unsigned int value)
 {
-	return new QTableWidgetItem(QString::number(value));
+	auto * item = new QTableWidgetItem(QString::number(value));
+	item->setFlags(Qt::NoItemFlags);
+	//item->setData(Qt::UserRole, QVariant::fromValue(value));
+	return item;
 }
 
 QTableWidgetItem * Inspector::addProperty(int value)
 {
-	return new QTableWidgetItem(QString::number(value));
+	auto * item = new QTableWidgetItem(QString::number(value));
+	item->setFlags(Qt::NoItemFlags);
+	//item->setData(Qt::UserRole, QVariant::fromValue(value));
+	return item;
 }
 
 QTableWidgetItem * Inspector::addProperty(bool value)
 {
-	return new QTableWidgetItem(value ? "TRUE" : "FALSE");
+	auto item = new QTableWidgetItem;
+	item->setFlags(Qt::ItemIsUserCheckable);
+	item->setCheckState(value ? Qt::Checked : Qt::Unchecked);
+	return item;
 }
 
 QTableWidgetItem * Inspector::addProperty(const std::string & value)
@@ -683,125 +814,74 @@ QTableWidgetItem * Inspector::addProperty(const std::string & value)
 	return addProperty(QString::fromStdString(value));
 }
 
+QTableWidgetItem * Inspector::addProperty(const TextIdentifier & value)
+{
+	return addProperty(VLC->generaltexth->translate(value.get()));
+}
+
+QTableWidgetItem * Inspector::addProperty(const MetaString & value)
+{
+	return addProperty(value.toString());
+}
+
 QTableWidgetItem * Inspector::addProperty(const QString & value)
 {
-	return new QTableWidgetItem(value);
+	auto * item = new QTableWidgetItem(value);
+	item->setFlags(Qt::NoItemFlags);
+	return item;
 }
 
 QTableWidgetItem * Inspector::addProperty(const int3 & value)
 {
-	return new QTableWidgetItem(QString("(%1, %2, %3)").arg(value.x, value.y, value.z));
+	auto * item = new QTableWidgetItem(QString("(%1, %2, %3)").arg(value.x, value.y, value.z));
+	item->setFlags(Qt::NoItemFlags);
+	return item;
 }
 
 QTableWidgetItem * Inspector::addProperty(const PlayerColor & value)
 {
-	auto str = QString("PLAYER %1").arg(value.getNum());
+	auto str = QObject::tr("UNFLAGGABLE");
 	if(value == PlayerColor::NEUTRAL)
-		str = "NEUTRAL";
-	if(value == PlayerColor::UNFLAGGABLE)
-		str = "UNFLAGGABLE";
-	return new QTableWidgetItem(str);
+		str = QObject::tr("neutral");
+	
+	if(value.isValidPlayer())
+		str = QString::fromStdString(GameConstants::PLAYER_COLOR_NAMES[value]);
+	
+	auto * item = new QTableWidgetItem(str);
+	item->setFlags(Qt::NoItemFlags);
+	item->setData(Qt::UserRole, QVariant::fromValue(value.getNum()));
+	return item;
 }
 
-QTableWidgetItem * Inspector::addProperty(const Res::ERes & value)
+QTableWidgetItem * Inspector::addProperty(const GameResID & value)
 {
-	QString str;
-	switch (value) {
-		case Res::ERes::WOOD:
-			str = "WOOD";
-			break;
-		case Res::ERes::ORE:
-			str = "ORE";
-			break;
-		case Res::ERes::SULFUR:
-			str = "SULFUR";
-			break;
-		case Res::ERes::GEMS:
-			str = "GEMS";
-			break;
-		case Res::ERes::MERCURY:
-			str = "MERCURY";
-			break;
-		case Res::ERes::CRYSTAL:
-			str = "CRYSTAL";
-			break;
-		case Res::ERes::GOLD:
-			str = "GOLD";
-			break;
-		default:
-			break;
-	}
-	return new QTableWidgetItem(str);
+	auto * item = new QTableWidgetItem(QString::fromStdString(GameConstants::RESOURCE_NAMES[value.toEnum()]));
+	item->setFlags(Qt::NoItemFlags);
+	item->setData(Qt::UserRole, QVariant::fromValue(value.getNum()));
+	return item;
 }
 
 QTableWidgetItem * Inspector::addProperty(CGCreature::Character value)
 {
-	QString str;
-	switch (value) {
-		case CGCreature::Character::COMPLIANT:
-			str = "COMPLIANT";
-			break;
-		case CGCreature::Character::FRIENDLY:
-			str = "FRIENDLY";
-			break;
-		case CGCreature::Character::AGRESSIVE:
-			str = "AGRESSIVE";
-			break;
-		case CGCreature::Character::HOSTILE:
-			str = "HOSTILE";
-			break;
-		case CGCreature::Character::SAVAGE:
-			str = "SAVAGE";
-			break;
-		default:
+	auto * item = new QTableWidgetItem;
+	item->setFlags(Qt::NoItemFlags);
+	item->setData(Qt::UserRole, QVariant::fromValue(int(value)));
+	
+	for(auto & i : CharacterIdentifiers)
+	{
+		if(i.second.toInt() == value)
+		{
+			item->setText(i.first);
 			break;
+		}
 	}
-	return new QTableWidgetItem(str);
-}
-
-QTableWidgetItem * Inspector::addProperty(CQuest::Emission value)
-{
-	QString str;
-	switch (value) {
-		case CQuest::Emission::MISSION_LEVEL:
-			str = "Reach level";
-			break;
-		case CQuest::Emission::MISSION_PRIMARY_STAT:
-			str = "Stats";
-			break;
-		case CQuest::Emission::MISSION_KILL_HERO:
-			str = "Kill hero";
-			break;
-		case CQuest::Emission::MISSION_KILL_CREATURE:
-			str = "Kill creature";
-			break;
-		case CQuest::Emission::MISSION_ART:
-			str = "Artifact";
-			break;
-		case CQuest::Emission::MISSION_ARMY:
-			str = "Army";
-			break;
-		case CQuest::Emission::MISSION_RESOURCES:
-			str = "Resources";
-			break;
-		case CQuest::Emission::MISSION_HERO:
-			str = "Hero";
-			break;
-		case CQuest::Emission::MISSION_PLAYER:
-			str = "Player";
-			break;
-		case CQuest::Emission::MISSION_KEYMASTER:
-			str = "Key master";
-			break;
-		default:
-			break;
-	}
-	return new QTableWidgetItem(str);
+	
+	return item;
 }
 
 //========================================================================
 
-Inspector::Inspector(CMap * m, CGObjectInstance * o, QTableWidget * t): obj(o), table(t), map(m)
+Inspector::Inspector(MapController & c, CGObjectInstance * o, QTableWidget * t): obj(o), table(t), controller(c)
 {
 }
 
@@ -809,13 +889,6 @@ Inspector::Inspector(CMap * m, CGObjectInstance * o, QTableWidget * t): obj(o),
  * Delegates
  */
 
-InspectorDelegate * InspectorDelegate::boolDelegate()
-{
-	auto * d = new InspectorDelegate;
-	d->options << "TRUE" << "FALSE";
-	return d;
-}
-
 QWidget * InspectorDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const
 {
 	return new QComboBox(parent);
@@ -825,7 +898,11 @@ void InspectorDelegate::setEditorData(QWidget *editor, const QModelIndex &index)
 {
 	if(QComboBox *ed = qobject_cast<QComboBox *>(editor))
 	{
-		ed->addItems(options);
+		for(auto & i : options)
+		{
+			ed->addItem(i.first);
+			ed->setItemData(ed->count() - 1, i.second);
+		}
 	}
 	else
 	{
@@ -840,7 +917,8 @@ void InspectorDelegate::setModelData(QWidget *editor, QAbstractItemModel *model,
 		if(!options.isEmpty())
 		{
 			QMap<int, QVariant> data;
-			data[0] = options[ed->currentIndex()];
+			data[Qt::DisplayRole] = options[ed->currentIndex()].first;
+			data[Qt::UserRole] = options[ed->currentIndex()].second;
 			model->setItemData(index, data);
 		}
 	}
@@ -849,4 +927,3 @@ void InspectorDelegate::setModelData(QWidget *editor, QAbstractItemModel *model,
 		QStyledItemDelegate::setModelData(editor, model, index);
 	}
 }
-
