@@ -4,12 +4,15 @@
 #include <QGraphicsScene>
 #include <QStandardItemModel>
 #include "mapcontroller.h"
-#include "../lib/Terrain.h"
 #include "resourceExtractor/ResourceConverter.h"
 
-class CMap;
+class ObjectBrowser;
 class ObjectBrowserProxyModel;
+
+VCMI_LIB_NAMESPACE_BEGIN
+class CMap;
 class CGObjectInstance;
+VCMI_LIB_NAMESPACE_END
 
 namespace Ui
 {
@@ -25,6 +28,12 @@ class MainWindow : public QMainWindow
 	const QString mainWindowSizeSetting = "MainWindow/Size";
 	const QString mainWindowPositionSetting = "MainWindow/Position";
 
+#ifdef ENABLE_QT_TRANSLATIONS
+	QTranslator translator;
+#endif
+	
+	std::unique_ptr<CMap> openMapInternal(const QString &);
+
 public:
     explicit MainWindow(QWidget *parent = nullptr);
     ~MainWindow();
@@ -34,7 +43,7 @@ public:
 	void saveMap();
 	bool openMap(const QString &);
 	
-	MapView * mapView();
+	//MapView * mapView();
 
 	void loadObjectsTree();
 
@@ -44,6 +53,8 @@ public:
 	
 	MapController controller;
 
+	void loadTranslation();
+
 private slots:
 	void on_actionOpen_triggered();
 
@@ -65,25 +76,16 @@ private slots:
 
 	void on_actionGrid_triggered(bool checked);
 
-	void on_toolBrush_clicked(bool checked);
-
-	void on_toolArea_clicked(bool checked);
-
 	void terrainButtonClicked(TerrainId terrain);
 	void roadOrRiverButtonClicked(ui8 type, bool isRoad);
+	void currentCoordinatesChanged(int x, int y);
 
-	void on_toolErase_clicked();
-
-	void on_terrainFilterCombo_currentTextChanged(const QString &arg1);
+	void on_terrainFilterCombo_currentIndexChanged(int index);
 
 	void on_filter_textChanged(const QString &arg1);
 
 	void on_actionFill_triggered();
 
-	void on_toolBrush2_clicked(bool checked);
-
-	void on_toolBrush4_clicked(bool checked);
-
 	void on_inspectorWidget_itemChanged(QTableWidgetItem *item);
 
 	void on_actionMapSettings_triggered();
@@ -98,6 +100,44 @@ private slots:
 	
 	void switchDefaultPlayer(const PlayerColor &);
 
+	void on_actionCut_triggered();
+
+	void on_actionCopy_triggered();
+
+	void on_actionPaste_triggered();
+
+	void on_actionExport_triggered();
+
+	void on_actionTranslations_triggered();
+	
+	void on_actionh3m_converter_triggered();
+
+	void on_actionLock_triggered();
+
+	void on_actionUnlock_triggered();
+
+	void on_actionZoom_in_triggered();
+
+	void on_actionZoom_out_triggered();
+
+	void on_actionZoom_reset_triggered();
+
+	void on_toolLine_toggled(bool checked);
+
+	void on_toolBrush2_toggled(bool checked);
+
+	void on_toolBrush_toggled(bool checked);
+
+	void on_toolBrush4_toggled(bool checked);
+
+	void on_toolLasso_toggled(bool checked);
+
+	void on_toolArea_toggled(bool checked);
+
+	void on_toolFill_toggled(bool checked);
+
+	void on_toolSelect_toggled(bool checked);
+
 public slots:
 
 	void treeViewSelected(const QModelIndex &selected, const QModelIndex &deselected);
@@ -134,12 +174,13 @@ private:
 	ObjectBrowserProxyModel * objectBrowser = nullptr;
 	QGraphicsScene * scenePreview;
 	
-	QString filename;
+	QString filename, lastSavingDir;
 	bool unsaved = false;
 
 	QStandardItemModel objectsModel;
 
 	int mapLevel = 0;
+	QRectF initialScale;
 
 	std::set<int> catalog;
 
