@@ -10,22 +10,28 @@
 
 #include "mapcontroller.h"
 
+#include "../lib/ArtifactUtils.h"
 #include "../lib/GameConstants.h"
+#include "../lib/mapObjectConstructors/AObjectTypeHandler.h"
+#include "../lib/mapObjectConstructors/CObjectClassesHandler.h"
+#include "../lib/mapObjects/ObjectTemplate.h"
 #include "../lib/mapping/CMapService.h"
 #include "../lib/mapping/CMap.h"
 #include "../lib/mapping/CMapEditManager.h"
-#include "../lib/Terrain.h"
-#include "../lib/mapObjects/CObjectClassesHandler.h"
-#include "../lib/rmg/ObstaclePlacer.h"
+#include "../lib/mapping/ObstacleProxy.h"
+#include "../lib/modding/CModHandler.h"
+#include "../lib/modding/CModInfo.h"
+#include "../lib/TerrainHandler.h"
 #include "../lib/CSkillHandler.h"
 #include "../lib/spells/CSpellHandler.h"
 #include "../lib/CHeroHandler.h"
+#include "../lib/serializer/CMemorySerializer.h"
 #include "mapview.h"
 #include "scenelayer.h"
 #include "maphandler.h"
 #include "mainwindow.h"
 #include "inspector/inspector.h"
-
+#include "VCMI_Lib.h"
 
 MapController::MapController(MainWindow * m): main(m)
 {
@@ -80,33 +86,35 @@ MinimapScene * MapController::miniScene(int level)
 
 void MapController::repairMap()
 {
-	//there might be extra skills, arts and spells not imported from map
-	if(VLC->skillh->getDefaultAllowed().size() > map()->allowedAbilities.size())
-	{
-		map()->allowedAbilities.resize(VLC->skillh->getDefaultAllowed().size());
-	}
-	if(VLC->arth->getDefaultAllowed().size() > map()->allowedArtifact.size())
-	{
-		map()->allowedArtifact.resize(VLC->arth->getDefaultAllowed().size());
-	}
-	if(VLC->spellh->getDefaultAllowed().size() > map()->allowedSpell.size())
-	{
-		map()->allowedSpell.resize(VLC->spellh->getDefaultAllowed().size());
-	}
-	if(VLC->heroh->getDefaultAllowed().size() > map()->allowedHeroes.size())
-	{
-		map()->allowedHeroes.resize(VLC->heroh->getDefaultAllowed().size());
-	}
+	repairMap(map());
+}
+
+void MapController::repairMap(CMap * map) const
+{
+	if(!map)
+		return;
+	
+	//make sure events/rumors has name to have proper identifiers
+	int emptyNameId = 1;
+	for(auto & e : map->events)
+		if(e.name.empty())
+			e.name = "event_" + std::to_string(emptyNameId++);
+	emptyNameId = 1;
+	for(auto & e : map->rumors)
+		if(e.name.empty())
+			e.name = "rumor_" + std::to_string(emptyNameId++);
 	
 	//fix owners for objects
-	for(auto obj : _map->objects)
+	auto allImpactedObjects(map->objects);
+	allImpactedObjects.insert(allImpactedObjects.end(), map->predefinedHeroes.begin(), map->predefinedHeroes.end());
+	for(auto obj : allImpactedObjects)
 	{
 		//setup proper names (hero name will be fixed later
 		if(obj->ID != Obj::HERO && obj->ID != Obj::PRISON && (obj->typeName.empty() || obj->subTypeName.empty()))
 		{
 			auto handler = VLC->objtypeh->getHandlerFor(obj->ID, obj->subID);
-			obj->typeName = handler->typeName;
-			obj->subTypeName = handler->subTypeName;
+			obj->typeName = handler->getTypeName();
+			obj->subTypeName = handler->getSubTypeName();
 		}
 		//fix flags
 		if(obj->getOwner() == PlayerColor::UNFLAGGABLE)
@@ -123,26 +131,24 @@ void MapController::repairMap()
 		//fix hero instance
 		if(auto * nih = dynamic_cast<CGHeroInstance*>(obj.get()))
 		{
-			map()->allowedHeroes.at(nih->subID) = true;
+			map->allowedHeroes.insert(nih->getHeroType());
 			auto type = VLC->heroh->objects[nih->subID];
 			assert(type->heroClass);
 			//TODO: find a way to get proper type name
 			if(obj->ID == Obj::HERO)
 			{
 				nih->typeName = "hero";
-				nih->subTypeName = type->heroClass->identifier;
+				nih->subTypeName = type->heroClass->getJsonKey();
 			}
 			if(obj->ID == Obj::PRISON)
 			{
 				nih->typeName = "prison";
 				nih->subTypeName = "prison";
+				nih->subID = 0;
 			}
 			
-			nih->type = type;
-			if(nih->name.empty())
-				nih->name = nih->type->name;
-			if(nih->biography.empty())
-				nih->biography = nih->type->biography;
+			if(obj->ID != Obj::RANDOM_HERO)
+				nih->type = type;
 			
 			if(nih->ID == Obj::HERO) //not prison
 				nih->appearance = VLC->objtypeh->getHandlerFor(Obj::HERO, type->heroClass->getIndex())->getTemplates().front();
@@ -150,15 +156,16 @@ void MapController::repairMap()
 			if(nih->spellbookContainsSpell(SpellID::PRESET))
 			{
 				nih->removeSpellFromSpellbook(SpellID::PRESET);
-			}
-			else
-			{
 				for(auto spellID : type->spells)
 					nih->addSpellToSpellbook(spellID);
 			}
-			//fix portrait
-			if(nih->portrait < 0 || nih->portrait == 255)
-				nih->portrait = type->imageIndex;
+			if(nih->spellbookContainsSpell(SpellID::SPELLBOOK_PRESET))
+			{
+				nih->removeSpellFromSpellbook(SpellID::SPELLBOOK_PRESET);
+				if(!nih->getArt(ArtifactPosition::SPELLBOOK) && type->haveSpellBook)
+					nih->putArtifact(ArtifactPosition::SPELLBOOK, ArtifactUtils::createNewArtifactInstance(ArtifactID::SPELLBOOK));
+			}
+			
 		}
 		//fix town instance
 		if(auto * tnh = dynamic_cast<CGTownInstance*>(obj.get()))
@@ -188,11 +195,11 @@ void MapController::repairMap()
 						out.push_back(spell->id);
 					}
 				}
-				auto a = CArtifactInstance::createScroll(*RandomGeneratorUtil::nextItem(out, CRandomGenerator::getDefault()));
+				auto a = ArtifactUtils::createScroll(*RandomGeneratorUtil::nextItem(out, CRandomGenerator::getDefault()));
 				art->storedArtifact = a;
 			}
 			else
-				map()->allowedArtifact.at(art->subID) = true;
+				map->allowedArtifact.insert(art->getArtifact());
 		}
 	}
 }
@@ -220,6 +227,18 @@ void MapController::setMap(std::unique_ptr<CMap> cmap)
 		}
 	);
 	_map->getEditManager()->getUndoManager().clearAll();
+
+	initObstaclePainters(_map.get());
+}
+
+void MapController::initObstaclePainters(CMap * map)
+{
+	for (auto terrain : VLC->terrainTypeHandler->objects)
+	{
+		auto terrainId = terrain->getId();
+		_obstaclePainters[terrainId] = std::make_unique<EditorObstaclePlacer>(map);
+		_obstaclePainters[terrainId]->collectPossibleObstacles(terrainId);
+	}
 }
 
 void MapController::sceneForceUpdate()
@@ -313,10 +332,11 @@ void MapController::commitObjectErase(int level)
 		return;
 	}
 
-	for (auto obj : selectedObjects)
+	for (auto & obj : selectedObjects)
 	{
 		//invalidate tiles under objects
-		_mapHandler->invalidate(_mapHandler->getTilesUnderObject(obj));
+		_mapHandler->removeObject(obj);
+		_scenes[level]->objectsView.setDirty(obj);
 	}
 
 	_scenes[level]->selectionObjectsView.clear();
@@ -328,6 +348,48 @@ void MapController::commitObjectErase(int level)
 	main->mapChanged();
 }
 
+void MapController::copyToClipboard(int level)
+{
+	_clipboard.clear();
+	_clipboardShiftIndex = 0;
+	auto selectedObjects = _scenes[level]->selectionObjectsView.getSelection();
+	for(auto * obj : selectedObjects)
+	{
+		assert(obj->pos.z == level);
+		_clipboard.push_back(CMemorySerializer::deepCopy(*obj));
+	}
+}
+
+void MapController::pasteFromClipboard(int level)
+{
+	_scenes[level]->selectionObjectsView.clear();
+	
+	auto shift = int3::getDirs()[_clipboardShiftIndex++];
+	if(_clipboardShiftIndex == int3::getDirs().size())
+		_clipboardShiftIndex = 0;
+	
+	for(auto & objUniquePtr : _clipboard)
+	{
+		auto * obj = CMemorySerializer::deepCopy(*objUniquePtr).release();
+		auto newPos = objUniquePtr->pos + shift;
+		if(_map->isInTheMap(newPos))
+			obj->pos = newPos;
+		obj->pos.z = level;
+		
+		Initializer init(obj, defaultPlayer);
+		_map->getEditManager()->insertObject(obj);
+		_scenes[level]->selectionObjectsView.selectObject(obj);
+		_mapHandler->invalidate(obj);
+	}
+	
+	_scenes[level]->objectsView.draw();
+	_scenes[level]->passabilityView.update();
+	_scenes[level]->selectionObjectsView.draw();
+	
+	_miniscenes[level]->updateViews();
+	main->mapChanged();
+}
+
 bool MapController::discardObject(int level) const
 {
 	_scenes[level]->selectionObjectsView.clear();
@@ -357,23 +419,29 @@ void MapController::commitObstacleFill(int level)
 		return;
 	
 	//split by zones
-	std::map<TerrainId, ObstacleProxy> terrainSelected;
+	for (auto & painter : _obstaclePainters)
+	{
+		painter.second->clearBlockedArea();
+	}
+	
 	for(auto & t : selection)
 	{
 		auto tl = _map->getTile(t);
 		if(tl.blocked || tl.visitable)
 			continue;
 		
-		terrainSelected[tl.terType->id].blockedArea.add(t);
+		auto terrain = tl.terType->getId();
+		_obstaclePainters[terrain]->addBlockedTile(t);
 	}
 	
-	for(auto & sel : terrainSelected)
+	for(auto & sel : _obstaclePainters)
 	{
-		sel.second.collectPossibleObstacles(sel.first);
-		sel.second.placeObstacles(_map.get(), CRandomGenerator::getDefault());
+		for(auto * o : sel.second->placeObstacles(CRandomGenerator::getDefault()))
+		{
+			_mapHandler->invalidate(o);
+			_scenes[level]->objectsView.setDirty(o);
+		}
 	}
-
-	_mapHandler->invalidateObjects();
 	
 	_scenes[level]->selectionTerrainView.clear();
 	_scenes[level]->selectionTerrainView.draw();
@@ -386,8 +454,8 @@ void MapController::commitObstacleFill(int level)
 
 void MapController::commitObjectChange(int level)
 {	
-	//for( auto * o : _scenes[level]->selectionObjectsView.getSelection())
-		//_mapHandler->invalidate(o);
+	for( auto * o : _scenes[level]->selectionObjectsView.getSelection())
+		_scenes[level]->objectsView.setDirty(o);
 	
 	_scenes[level]->objectsView.draw();
 	_scenes[level]->selectionObjectsView.draw();
@@ -416,9 +484,8 @@ void MapController::commitObjectShift(int level)
 			pos.z = level;
 			pos.x += shift.x(); pos.y += shift.y();
 			
-			auto prevPositions = _mapHandler->getTilesUnderObject(obj);
+			_scenes[level]->objectsView.setDirty(obj); //set dirty before movement
 			_map->getEditManager()->moveObject(obj, pos);
-			_mapHandler->invalidate(prevPositions);
 			_mapHandler->invalidate(obj);
 		}
 	}
@@ -456,6 +523,7 @@ void MapController::commitObjectCreate(int level)
 	
 	_map->getEditManager()->insertObject(newObj);
 	_mapHandler->invalidate(newObj);
+	_scenes[level]->objectsView.setDirty(newObj);
 	
 	_scenes[level]->selectionObjectsView.newObject = nullptr;
 	_scenes[level]->selectionObjectsView.shift = QPoint(0, 0);
@@ -501,7 +569,7 @@ void MapController::undo()
 {
 	_map->getEditManager()->getUndoManager().undo();
 	resetMapHandler();
-	sceneForceUpdate();
+	sceneForceUpdate(); //TODO: use smart invalidation (setDirty)
 	main->mapChanged();
 }
 
@@ -509,6 +577,39 @@ void MapController::redo()
 {
 	_map->getEditManager()->getUndoManager().redo();
 	resetMapHandler();
-	sceneForceUpdate();
+	sceneForceUpdate(); //TODO: use smart invalidation (setDirty)
 	main->mapChanged();
 }
+
+ModCompatibilityInfo MapController::modAssessmentAll()
+{
+	ModCompatibilityInfo result;
+	for(auto primaryID : VLC->objtypeh->knownObjects())
+	{
+		for(auto secondaryID : VLC->objtypeh->knownSubObjects(primaryID))
+		{
+			auto handler = VLC->objtypeh->getHandlerFor(primaryID, secondaryID);
+			auto modName = QString::fromStdString(handler->getJsonKey()).split(":").at(0).toStdString();
+			if(modName != "core")
+				result[modName] = VLC->modh->getModInfo(modName).getVerificationInfo();
+		}
+	}
+	return result;
+}
+
+ModCompatibilityInfo MapController::modAssessmentMap(const CMap & map)
+{
+	ModCompatibilityInfo result;
+	for(auto obj : map.objects)
+	{
+		if(obj->ID == Obj::HERO)
+			continue; //stub! 
+		
+		auto handler = obj->getObjectHandler();
+		auto modName = QString::fromStdString(handler->getJsonKey()).split(":").at(0).toStdString();
+		if(modName != "core")
+			result[modName] = VLC->modh->getModInfo(modName).getVerificationInfo();
+	}
+	//TODO: terrains?
+	return result;
+}
