@@ -15,6 +15,7 @@
 #include "../lib/rmg/CMapGenerator.h"
 #include "../lib/VCMI_Lib.h"
 #include "../lib/mapping/CMapEditManager.h"
+#include "../lib/mapping/MapFormat.h"
 #include "../lib/CGeneralTextHandler.h"
 
 #include "windownewmap.h"
@@ -31,6 +32,28 @@ WindowNewMap::WindowNewMap(QWidget *parent) :
 	setAttribute(Qt::WA_DeleteOnClose);
 
 	setWindowModality(Qt::ApplicationModal);
+	
+	for(auto * combo : {ui->humanCombo, ui->cpuCombo, ui->humanTeamsCombo, ui->cpuTeamsCombo})
+		combo->clear();
+	
+	//prepare human players combo box
+	for(int i = 0; i <= PlayerColor::PLAYER_LIMIT_I; ++i)
+	{
+		ui->humanCombo->addItem(!i ? randomString : QString::number(players.at(i)));
+		ui->humanCombo->setItemData(i, QVariant(players.at(i)));
+		
+		ui->cpuCombo->addItem(!i ? randomString : QString::number(cpuPlayers.at(i)));
+		ui->cpuCombo->setItemData(i, QVariant(cpuPlayers.at(i)));
+		
+		ui->humanTeamsCombo->addItem(!i ? randomString : QString::number(cpuPlayers.at(i)));
+		ui->humanTeamsCombo->setItemData(i, QVariant(cpuPlayers.at(i)));
+		
+		ui->cpuTeamsCombo->addItem(!i ? randomString : QString::number(cpuPlayers.at(i)));
+		ui->cpuTeamsCombo->setItemData(i, QVariant(cpuPlayers.at(i)));
+	}
+	
+	for(auto * combo : {ui->humanCombo, ui->cpuCombo, ui->humanTeamsCombo, ui->cpuTeamsCombo})
+		combo->setCurrentIndex(0);
 
 	loadUserSettings();
 
@@ -67,6 +90,11 @@ void WindowNewMap::loadUserSettings()
 	{
 		ui->heightTxt->setText(height.toString());
 	}
+	for(auto & sz : mapSizes)
+	{
+		if(sz.second.first == width.toInt() && sz.second.second == height.toInt())
+			ui->sizeCombo->setCurrentIndex(sz.first);
+	}
 	auto twoLevel = s.value(newMapTwoLevel);
 	if (twoLevel.isValid())
 	{
@@ -87,8 +115,17 @@ void WindowNewMap::loadUserSettings()
 	{
 		ui->cpuCombo->setCurrentIndex(cpuPlayers.toInt());
 	}
-	//TODO: teams when implemented
-
+	auto teams = s.value(newMapHumanTeams);
+	if(teams.isValid())
+	{
+		ui->humanTeamsCombo->setCurrentIndex(teams.toInt());
+	}
+	auto cputeams = s.value(newMapCpuTeams);
+	if(cputeams.isValid())
+	{
+		ui->cpuTeamsCombo->setCurrentIndex(cputeams.toInt());
+	}
+	
 	auto waterContent = s.value(newMapWaterContent);
 	if (waterContent.isValid())
 	{
@@ -150,7 +187,8 @@ void WindowNewMap::saveUserSettings()
 
 	s.setValue(newMapPlayers,ui->humanCombo->currentIndex());
 	s.setValue(newMapCpuPlayers,ui->cpuCombo->currentIndex());
-	//TODO: teams when implemented
+	s.setValue(newMapHumanTeams, ui->humanTeamsCombo->currentIndex());
+	s.setValue(newMapCpuTeams, ui->cpuTeamsCombo->currentIndex());
 
 	EWaterContent::EWaterContent water = EWaterContent::RANDOM;
 	if(ui->waterOpt1->isChecked())
@@ -202,7 +240,7 @@ std::unique_ptr<CMap> generateEmptyMap(CMapGenOptions & options)
 	map->initTerrain();
 	map->getEditManager()->clearTerrain(&CRandomGenerator::getDefault());
 
-	return std::move(map);
+	return map;
 }
 
 void WindowNewMap::on_okButton_clicked()
@@ -235,7 +273,7 @@ void WindowNewMap::on_okButton_clicked()
 		//verify map template
 		if(mapGenOptions.getPossibleTemplates().empty())
 		{
-			QMessageBox::warning(this, "No template", "No template for parameters scecified. Random map cannot be generated.");
+			QMessageBox::warning(this, tr("No template"), tr("No template for parameters scecified. Random map cannot be generated."));
 			return;
 		}
 		
@@ -255,7 +293,7 @@ void WindowNewMap::on_okButton_clicked()
 		}
 		catch(const std::exception & e)
 		{
-			QMessageBox::critical(this, "RMG failure", e.what());
+			QMessageBox::critical(this, tr("RMG failure"), e.what());
 		}
 	}
 	else
@@ -264,7 +302,7 @@ void WindowNewMap::on_okButton_clicked()
 		nmap = f.get();
 	}
 	
-
+	nmap->mods = MapController::modAssessmentAll();
 	static_cast<MainWindow*>(parent())->controller.setMap(std::move(nmap));
 	static_cast<MainWindow*>(parent())->initializeMap(true);
 	close();
@@ -272,16 +310,8 @@ void WindowNewMap::on_okButton_clicked()
 
 void WindowNewMap::on_sizeCombo_activated(int index)
 {
-	std::map<int, std::pair<int, int>> sizes
-	{
-		{0, {36, 36}},
-		{1, {72, 72}},
-		{2, {108, 108}},
-		{3, {144, 144}},
-	};
-
-	ui->widthTxt->setText(QString::number(sizes[index].first));
-	ui->heightTxt->setText(QString::number(sizes[index].second));
+	ui->widthTxt->setText(QString::number(mapSizes.at(index).first));
+	ui->heightTxt->setText(QString::number(mapSizes.at(index).second));
 }
 
 
@@ -295,54 +325,61 @@ void WindowNewMap::on_twoLevelCheck_stateChanged(int arg1)
 
 void WindowNewMap::on_humanCombo_activated(int index)
 {
-	int humans = players.at(index);
-	if(humans > playerLimit)
+	int humans = ui->humanCombo->currentData().toInt();
+	if(humans > PlayerColor::PLAYER_LIMIT_I)
 	{
-		humans = playerLimit;
+		humans = PlayerColor::PLAYER_LIMIT_I;
 		ui->humanCombo->setCurrentIndex(humans);
-		return;
 	}
 
-	mapGenOptions.setPlayerCount(humans);
+	mapGenOptions.setHumanOrCpuPlayerCount(humans);
 
 	int teams = mapGenOptions.getTeamCount();
 	if(teams > humans - 1)
 	{
-		teams = humans - 1;
-		//TBD
+		teams = humans > 0 ? humans - 1 : CMapGenOptions::RANDOM_SIZE;
+		ui->humanTeamsCombo->setCurrentIndex(teams + 1); //skip one element because first is random
 	}
 
 	int cpu = mapGenOptions.getCompOnlyPlayerCount();
-	if(cpu > playerLimit - humans)
+	if(cpu > PlayerColor::PLAYER_LIMIT_I - humans)
 	{
-		cpu = playerLimit - humans;
-		ui->cpuCombo->setCurrentIndex(cpu + 1);
+		cpu = PlayerColor::PLAYER_LIMIT_I - humans;
+		ui->cpuCombo->setCurrentIndex(cpu + 1); //skip one element because first is random
 	}
 
 	int cpuTeams = mapGenOptions.getCompOnlyTeamCount(); //comp only players - 1
 	if(cpuTeams > cpu - 1)
 	{
-		cpuTeams = cpu - 1;
-		//TBD
+		cpuTeams = cpu > 0 ? cpu - 1 : CMapGenOptions::RANDOM_SIZE;
+		ui->cpuTeamsCombo->setCurrentIndex(cpuTeams + 1); //skip one element because first is random
 	}
 
-	//void setMapTemplate(const CRmgTemplate * value);
 	updateTemplateList();
 }
 
 
 void WindowNewMap::on_cpuCombo_activated(int index)
 {
-	int humans = mapGenOptions.getPlayerCount();
-	int cpu = cpuPlayers.at(index);
-	if(cpu > playerLimit - humans)
+	int humans = mapGenOptions.getHumanOrCpuPlayerCount();
+	int cpu = ui->cpuCombo->currentData().toInt();
+
+	// FIXME: Use mapGenOption method only to calculate actual number of players for current template
+	if(cpu > PlayerColor::PLAYER_LIMIT_I - humans)
 	{
-		cpu = playerLimit - humans;
-		ui->cpuCombo->setCurrentIndex(cpu + 1);
-		return;
+		cpu = PlayerColor::PLAYER_LIMIT_I - humans;
+		ui->cpuCombo->setCurrentIndex(cpu + 1); //skip one element because first is random
+	}
+	
+	mapGenOptions.setCompOnlyPlayerCount(cpu);
+	
+	int cpuTeams = mapGenOptions.getCompOnlyTeamCount(); //comp only players - 1
+	if(cpuTeams > cpu - 1)
+	{
+		cpuTeams = cpu > 0 ? cpu - 1 : CMapGenOptions::RANDOM_SIZE;
+		ui->cpuTeamsCombo->setCurrentIndex(cpuTeams + 1); //skip one element because first is random
 	}
 
-	mapGenOptions.setCompOnlyPlayerCount(cpu);
 	updateTemplateList();
 }
 
@@ -417,3 +454,33 @@ void WindowNewMap::on_checkSeed_toggled(bool checked)
 	ui->lineSeed->setEnabled(checked);
 }
 
+
+void WindowNewMap::on_humanTeamsCombo_activated(int index)
+{
+	int humans = mapGenOptions.getHumanOrCpuPlayerCount();
+	int teams = ui->humanTeamsCombo->currentData().toInt();
+	if(teams >= humans)
+	{
+		teams = humans > 0 ? humans - 1 : CMapGenOptions::RANDOM_SIZE;
+		ui->humanTeamsCombo->setCurrentIndex(teams + 1); //skip one element because first is random
+	}
+
+	mapGenOptions.setTeamCount(teams);
+	updateTemplateList();
+}
+
+
+void WindowNewMap::on_cpuTeamsCombo_activated(int index)
+{
+	int cpu = mapGenOptions.getCompOnlyPlayerCount();
+	int cpuTeams = ui->cpuTeamsCombo->currentData().toInt();
+	if(cpuTeams >= cpu)
+	{
+		cpuTeams = cpu > 0 ? cpu - 1 : CMapGenOptions::RANDOM_SIZE;
+		ui->cpuTeamsCombo->setCurrentIndex(cpuTeams + 1); //skip one element because first is random
+	}
+
+	mapGenOptions.setCompOnlyTeamCount(cpuTeams);
+	updateTemplateList();
+}
+
