@@ -10,7 +10,9 @@
 
 #pragma once
 
-#include "../../lib/HeroBonus.h"
+#include "../../lib/bonuses/BonusList.h"
+#include "../../lib/bonuses/Bonus.h"
+#include "../../lib/bonuses/IBonusBearer.h"
 
 
 class BonusBearerMock : public IBonusBearer
