@@ -16,25 +16,31 @@
 class IBattleInfoCallbackMock : public IBattleInfoCallback
 {
 public:
+#if SCRIPTING_ENABLED
 	MOCK_CONST_METHOD0(getContextPool, scripting::Pool *());
+#endif
 	MOCK_CONST_METHOD0(battleTerrainType, TerrainId());
 	MOCK_CONST_METHOD0(battleGetBattlefieldType, BattleField());
 
-	MOCK_CONST_METHOD0(battleIsFinished, boost::optional<int>());
+	MOCK_CONST_METHOD0(battleIsFinished, std::optional<int>());
 
 	MOCK_CONST_METHOD0(battleTacticDist, si8());
 	MOCK_CONST_METHOD0(battleGetTacticsSide, si8());
 
 	MOCK_CONST_METHOD0(battleNextUnitId, uint32_t());
 
-	MOCK_CONST_METHOD1(battleGetUnitsIf, battle::Units(battle::UnitFilter));
+	MOCK_CONST_METHOD1(battleGetUnitsIf, battle::Units(const battle::UnitFilter &));
 
 	MOCK_CONST_METHOD1(battleGetUnitByID, const battle::Unit *(uint32_t));
 	MOCK_CONST_METHOD2(battleGetUnitByPos, const battle::Unit *(BattleHex, bool));
 	MOCK_CONST_METHOD0(battleActiveUnit, const battle::Unit *());
 
+	MOCK_CONST_METHOD0(getBonusBearer, IBonusBearer*());
+	MOCK_CONST_METHOD0(getBattle, IBattleInfo*());
+	MOCK_CONST_METHOD0(getPlayerID, std::optional<PlayerColor>());
+
 	MOCK_CONST_METHOD2(battleGetAllObstaclesOnPos, std::vector<std::shared_ptr<const CObstacleInstance>>(BattleHex, bool));
-	MOCK_CONST_METHOD1(getAllAffectedObstaclesByStack, std::vector<std::shared_ptr<const CObstacleInstance>>(const battle::Unit *));
+	MOCK_CONST_METHOD2(getAllAffectedObstaclesByStack, std::vector<std::shared_ptr<const CObstacleInstance>>(const battle::Unit *, const std::set<BattleHex> &));
 
 };
 
