@@ -16,12 +16,16 @@ class IGameInfoCallbackMock : public IGameInfoCallback
 {
 public:
 	//various
-	MOCK_CONST_METHOD1(getDate, int(Date::EDateType));
-	MOCK_CONST_METHOD2(isAllowed, bool(int32_t, int32_t));
+	MOCK_CONST_METHOD1(getDate, int(Date));
+
+	MOCK_CONST_METHOD1(isAllowed, bool(SpellID));
+	MOCK_CONST_METHOD1(isAllowed, bool(ArtifactID));
+	MOCK_CONST_METHOD1(isAllowed, bool(SecondarySkill));
 
 	//player
 	MOCK_CONST_METHOD1(getPlayer, const Player *(PlayerColor));
 	MOCK_CONST_METHOD0(getLocalPlayer, PlayerColor());
+	MOCK_CONST_METHOD0(getPlayerID, std::optional<PlayerColor>());
 
 	//hero
 	MOCK_CONST_METHOD1(getHero, const CGHeroInstance *(ObjectInstanceID));
